@@ -30,7 +30,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <sys/types.h>
 #include <setjmp.h>
 #include <jpeglib.h>
 #include <endian.h>
@@ -235,40 +234,43 @@ image_get_jpeg_resolution(const char * path, int * width, int * height)
 {
 	FILE *img;
 	unsigned char buf[8];
-	u_int16_t offset, h, w;
+	uint16_t offset, h, w;
 	int ret = 1;
+	size_t nread;
 	long size;
 	
 
 	img = fopen(path, "r");
 	if( !img )
-		return(-1);
+		return -1;
 
 	fseek(img, 0, SEEK_END);
 	size = ftell(img);
 	rewind(img);
 
-	fread(&buf, 2, 1, img);
-	if( (buf[0] != 0xFF) || (buf[1] != 0xD8) )
+	nread = fread(&buf, 2, 1, img);
+	if( (nread < 1) || (buf[0] != 0xFF) || (buf[1] != 0xD8) )
 	{
 		fclose(img);
-		return(-1);
+		return -1;
 	}
 	memset(&buf, 0, sizeof(buf));
 
 	while( ftell(img) < size )
 	{
-		while( buf[0] != 0xFF && !feof(img) )
-			fread(&buf, 1, 1, img);
+		while( nread > 0 && buf[0] != 0xFF && !feof(img) )
+			nread = fread(&buf, 1, 1, img);
 
-		while( buf[0] == 0xFF && !feof(img) )
-			fread(&buf, 1, 1, img);
+		while( nread > 0 && buf[0] == 0xFF && !feof(img) )
+			nread = fread(&buf, 1, 1, img);
 
 		if( (buf[0] >= 0xc0) && (buf[0] <= 0xc3) )
 		{
-			fread(&buf, 7, 1, img);
+			nread = fread(&buf, 7, 1, img);
 			*width = 0;
 			*height = 0;
+			if( nread < 1 )
+				break;
 			memcpy(&h, buf+3, 2);
 			*height = SWAP16(h);
 			memcpy(&w, buf+5, 2);
@@ -279,7 +281,9 @@ image_get_jpeg_resolution(const char * path, int * width, int * height)
 		else
 		{
 			offset = 0;
-			fread(&buf, 2, 1, img);
+			nread = fread(&buf, 2, 1, img);
+			if( nread < 1 )
+				break;
 			memcpy(&offset, buf, 2);
 			offset = SWAP16(offset) - 2;
 			if( fseek(img, offset, SEEK_CUR) == -1 )
@@ -296,17 +300,18 @@ image_get_jpeg_date_xmp(const char * path, char ** date)
 	FILE *img;
 	unsigned char buf[8];
 	char *data = NULL, *newdata;
-	u_int16_t offset;
+	uint16_t offset;
 	struct NameValueParserData xml;
 	char * exif;
 	int ret = 1;
+	size_t nread;
 
 	img = fopen(path, "r");
 	if( !img )
 		return(-1);
 
-	fread(&buf, 2, 1, img);
-	if( (buf[0] != 0xFF) || (buf[1] != 0xD8) )
+	nread = fread(&buf, 2, 1, img);
+	if( (nread < 1) || (buf[0] != 0xFF) || (buf[1] != 0xD8) )
 	{
 		fclose(img);
 		return(-1);
@@ -315,11 +320,11 @@ image_get_jpeg_date_xmp(const char * path, char ** date)
 
 	while( !feof(img) )
 	{
-		while( buf[0] != 0xFF && !feof(img) )
-			fread(&buf, 1, 1, img);
+		while( nread > 0 && buf[0] != 0xFF && !feof(img) )
+			nread = fread(&buf, 1, 1, img);
 
-		while( buf[0] == 0xFF && !feof(img) )
-			fread(&buf, 1, 1, img);
+		while( nread > 0 && buf[0] == 0xFF && !feof(img) )
+			nread = fread(&buf, 1, 1, img);
 
 		if( feof(img) )
 			break;
@@ -327,7 +332,9 @@ image_get_jpeg_date_xmp(const char * path, char ** date)
 		if( buf[0] == 0xE1 ) // APP1 marker
 		{
 			offset = 0;
-			fread(&buf, 2, 1, img);
+			nread = fread(&buf, 2, 1, img);
+			if( nread < 1 )
+				break;
 			memcpy(&offset, buf, 2);
 			offset = SWAP16(offset) - 2;
 
@@ -342,7 +349,9 @@ image_get_jpeg_date_xmp(const char * path, char ** date)
 				break;
 			data = newdata;
 
-			fread(data, 29, 1, img);
+			nread = fread(data, 29, 1, img);
+			if( nread < 1 )
+				break;
 			offset -= 29;
 			if( strcmp(data, "http://ns.adobe.com/xap/1.0/") != 0 )
 			{
@@ -354,7 +363,9 @@ image_get_jpeg_date_xmp(const char * path, char ** date)
 			if( !newdata )
 				break;
 			data = newdata;
-			fread(data, offset, 1, img);
+			nread = fread(data, offset, 1, img);
+			if( nread < 1 )
+				break;
 
 			ParseNameValue(data, offset, &xml);
 			exif = GetValueFromNameValueList(&xml, "DateTimeOriginal");
@@ -373,7 +384,9 @@ image_get_jpeg_date_xmp(const char * path, char ** date)
 		else
 		{
 			offset = 0;
-			fread(&buf, 2, 1, img);
+			nread = fread(&buf, 2, 1, img);
+			if( nread < 1 )
+				break;
 			memcpy(&offset, buf, 2);
 			offset = SWAP16(offset) - 2;
 			fseek(img, offset, SEEK_CUR);
@@ -407,7 +420,7 @@ image_new(int32_t width, int32_t height)
 }
 
 image_s *
-image_new_from_jpeg(const char * path, int is_file, const char * buf, int size, int scale)
+image_new_from_jpeg(const char * path, int is_file, const char * buf, int size, int scale, int rotate)
 {
 	image_s *vimage;
 	FILE  *file = NULL;
@@ -417,7 +430,6 @@ image_new_from_jpeg(const char * path, int is_file, const char * buf, int size,
 	int maxbuf;
 	struct jpeg_error_mgr pub;
 
-
 	cinfo.err = jpeg_std_error(&pub);
 	pub.error_exit = libjpeg_error_handler;
 	jpeg_create_decompress(&cinfo);
@@ -447,7 +459,7 @@ image_new_from_jpeg(const char * path, int is_file, const char * buf, int size,
 	jpeg_start_decompress(&cinfo);
 	w = cinfo.output_width;
 	h = cinfo.output_height;
-	vimage = image_new(w, h);
+	vimage = (rotate & (ROTATE_90|ROTATE_270)) ? image_new(h, w) : image_new(w, h);
 	if(!vimage)
 	{
 		jpeg_destroy_decompress(&cinfo);
@@ -481,8 +493,9 @@ image_new_from_jpeg(const char * path, int is_file, const char * buf, int size,
 	maxbuf = vimage->width * vimage->height;
 	if(cinfo.output_components == 3)
 	{
+		int rx, ry;
 		ofs = 0;
-		if((ptr = (unsigned char *)malloc(w * 3 * cinfo.rec_outbuf_height + 8)) == NULL)
+		if((ptr = malloc(w * 3 * cinfo.rec_outbuf_height + 16)) == NULL)
 		{
 			DPRINTF(E_WARN, L_METADATA, "malloc failed\n");
 			image_free(vimage);
@@ -493,6 +506,7 @@ image_new_from_jpeg(const char * path, int is_file, const char * buf, int size,
 
 		for(y = 0; y < h; y += cinfo.rec_outbuf_height)
 		{
+			ry = (rotate & (ROTATE_90|ROTATE_180)) ? (y - h + 1) * -1 : y;
 			for(i = 0; i < cinfo.rec_outbuf_height; i++)
 			{
 				line[i] = ptr + (w * 3 * i);
@@ -500,11 +514,10 @@ image_new_from_jpeg(const char * path, int is_file, const char * buf, int size,
 			jpeg_read_scanlines(&cinfo, line, cinfo.rec_outbuf_height);
 			for(x = 0; x < w * cinfo.rec_outbuf_height; x++)
 			{
+				rx = (rotate & (ROTATE_180|ROTATE_270)) ? (x - w + 1) * -1 : x;
+				ofs = (rotate & (ROTATE_90|ROTATE_270)) ? ry + (rx * h) : rx + (ry * w);
 				if( ofs < maxbuf )
-				{
 					vimage->buf[ofs] = COL(ptr[x + x + x], ptr[x + x + x + 1], ptr[x + x + x + 2]);
-					ofs++;
-				}
 			}
 		}
 		free(ptr);
@@ -514,7 +527,7 @@ image_new_from_jpeg(const char * path, int is_file, const char * buf, int size,
 		ofs = 0;
 		for(i = 0; i < cinfo.rec_outbuf_height; i++)
 		{
-			if((line[i] = (unsigned char *)malloc(w)) == NULL)
+			if((line[i] = malloc(w)) == NULL)
 			{
 				int t = 0;
 
@@ -590,19 +603,19 @@ image_upsize(image_s * pdest, image_s * psrc, int32_t width, int32_t height)
 
 			x_dist = rx - ((float)((int32_t)rx));
 			y_dist = ry - ((float)((int32_t)ry));
-			vcol = COL_FULL( (u_int8_t)((COL_RED(vcol1)*(1.0-x_dist)
+			vcol = COL_FULL( (uint8_t)((COL_RED(vcol1)*(1.0-x_dist)
 			                  + COL_RED(vcol2)*(x_dist))*(1.0-y_dist)
 			                  + (COL_RED(vcol3)*(1.0-x_dist)
 			                  + COL_RED(vcol4)*(x_dist))*(y_dist)),
-			                 (u_int8_t)((COL_GREEN(vcol1)*(1.0-x_dist)
+			                 (uint8_t)((COL_GREEN(vcol1)*(1.0-x_dist)
 			                  + COL_GREEN(vcol2)*(x_dist))*(1.0-y_dist)
 			                  + (COL_GREEN(vcol3)*(1.0-x_dist)
 			                  + COL_GREEN(vcol4)*(x_dist))*(y_dist)),
-			                 (u_int8_t)((COL_BLUE(vcol1)*(1.0-x_dist)
+			                 (uint8_t)((COL_BLUE(vcol1)*(1.0-x_dist)
 			                  + COL_BLUE(vcol2)*(x_dist))*(1.0-y_dist)
 			                  + (COL_BLUE(vcol3)*(1.0-x_dist)
 			                  + COL_BLUE(vcol4)*(x_dist))*(y_dist)),
-			                 (u_int8_t)((COL_ALPHA(vcol1)*(1.0-x_dist)
+			                 (uint8_t)((COL_ALPHA(vcol1)*(1.0-x_dist)
 			                  + COL_ALPHA(vcol2)*(x_dist))*(1.0-y_dist)
 			                  + (COL_ALPHA(vcol3)*(1.0-x_dist)
 			                  + COL_ALPHA(vcol4)*(x_dist))*(y_dist))
@@ -752,7 +765,7 @@ image_downsize(image_s * pdest, image_s * psrc, int32_t width, int32_t height)
 			alpha = (alpha > 255.0)? 255.0 : ((alpha < 0.0)? 0.0:alpha);
 #endif
 			put_pix_alpha_replace(pdest, vx, vy,
-					      COL_FULL((u_int8_t)red, (u_int8_t)green, (u_int8_t)blue, (u_int8_t)alpha));
+					      COL_FULL((uint8_t)red, (uint8_t)green, (uint8_t)blue, (uint8_t)alpha));
 		}
 	}
 }
@@ -806,9 +819,9 @@ image_save_to_jpeg_buf(image_s * pimage, int * size)
 	{
 		for(x = 0; x < pimage->width; x++)
 		{
-			data[x + x + x]   = COL_RED(pimage->buf[i]);
-			data[x + x + x + 1] = COL_GREEN(pimage->buf[i]);
-			data[x + x + x + 2] = COL_BLUE(pimage->buf[i]);
+			data[x * 3]     = COL_RED(pimage->buf[i]);
+			data[x * 3 + 1] = COL_GREEN(pimage->buf[i]);
+			data[x * 3 + 2] = COL_BLUE(pimage->buf[i]);
 			i++;
 		}
 		row_pointer[0] = (unsigned char *)data;
