@@ -1,19 +1,19 @@
-/*  MiniDLNA media server
- *  Copyright (C) 2008-2010  Justin Maggard
+/* MiniDLNA media server
+ * Copyright (C) 2008-2010  Justin Maggard
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This file is part of MiniDLNA.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * MiniDLNA is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * MiniDLNA is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with MiniDLNA. If not, see <http://www.gnu.org/licenses/>.
  */
 #include "config.h"
 #include <stdio.h>
@@ -143,20 +143,33 @@ int
 inotify_create_watches(int fd)
 {
 	FILE * max_watches;
-	unsigned int num_watches, watch_limit = 8192;
+	unsigned int num_watches = 0, watch_limit;
 	char **result;
 	int i, rows = 0;
 	struct media_dir_s * media_path;
 
-	i = sql_get_int_field(db, "SELECT count(*) from DETAILS where SIZE is NULL and PATH is not NULL");
-	num_watches = (i < 0) ? 0 : i;
+	for( media_path = media_dirs; media_path != NULL; media_path = media_path->next )
+	{
+		DPRINTF(E_DEBUG, L_INOTIFY, "Add watch to %s\n", media_path->path);
+		add_watch(fd, media_path->path);
+		num_watches++;
+	}
+	sql_get_table(db, "SELECT PATH from DETAILS where MIME is NULL and PATH is not NULL", &result, &rows, NULL);
+	for( i=1; i <= rows; i++ )
+	{
+		DPRINTF(E_DEBUG, L_INOTIFY, "Add watch to %s\n", result[i]);
+		add_watch(fd, result[i]);
+		num_watches++;
+	}
+	sqlite3_free_table(result);
 		
 	max_watches = fopen("/proc/sys/fs/inotify/max_user_watches", "r");
 	if( max_watches )
 	{
-		fscanf(max_watches, "%u", &watch_limit);
+		if( fscanf(max_watches, "%10u", &watch_limit) < 1 )
+			watch_limit = 8192;
 		fclose(max_watches);
-		if( (watch_limit < DESIRED_WATCH_LIMIT) || (watch_limit < (num_watches*3/4)) )
+		if( (watch_limit < DESIRED_WATCH_LIMIT) || (watch_limit < (num_watches*4/3)) )
 		{
 			max_watches = fopen("/proc/sys/fs/inotify/max_user_watches", "w");
 			if( max_watches )
@@ -189,20 +202,6 @@ inotify_create_watches(int fd)
 		                        "Hopefully it is enough to cover %u current directories plus any new ones added.\n", num_watches);
 	}
 
-	media_path = media_dirs;
-	while( media_path )
-	{
-		add_watch(fd, media_path->path);
-		media_path = media_path->next;
-	}
-	sql_get_table(db, "SELECT PATH from DETAILS where SIZE is NULL and PATH is not NULL", &result, &rows, NULL);
-	for( i=1; i <= rows; i++ )
-	{
-		DPRINTF(E_DEBUG, L_INOTIFY, "Add watch to %s\n", result[i]);
-		add_watch(fd, result[i]);
-	}
-	sqlite3_free_table(result);
-
 	return rows;
 }
 
@@ -231,26 +230,30 @@ int add_dir_watch(int fd, char * path, char * filename)
 {
 	DIR *ds;
 	struct dirent *e;
-	char *buf;
+	char *dir;
+	char buf[PATH_MAX];
 	int wd;
 	int i = 0;
 
 	if( filename )
-		asprintf(&buf, "%s/%s", path, filename);
+	{
+		snprintf(buf, sizeof(buf), "%s/%s", path, filename);
+		dir = buf;
+	}
 	else
-		buf = strdup(path);
+		dir = path;
 
-	wd = add_watch(fd, buf);
+	wd = add_watch(fd, dir);
 	if( wd == -1 )
 	{
 		DPRINTF(E_ERROR, L_INOTIFY, "add_watch() [%s]\n", strerror(errno));
 	}
 	else
 	{
-		DPRINTF(E_INFO, L_INOTIFY, "Added watch to %s [%d]\n", buf, wd);
+		DPRINTF(E_INFO, L_INOTIFY, "Added watch to %s [%d]\n", dir, wd);
 	}
 
-	ds = opendir(buf);
+	ds = opendir(dir);
 	if( ds != NULL )
 	{
 		while( (e = readdir(ds)) )
@@ -259,8 +262,8 @@ int add_dir_watch(int fd, char * path, char * filename)
 			    strcmp(e->d_name, "..") == 0 )
 				continue;
 			if( (e->d_type == DT_DIR) ||
-			    (e->d_type == DT_UNKNOWN && resolve_unknown_type(buf, NO_MEDIA) == TYPE_DIR) )
-				i += add_dir_watch(fd, buf, e->d_name);
+			    (e->d_type == DT_UNKNOWN && resolve_unknown_type(dir, NO_MEDIA) == TYPE_DIR) )
+				i += add_dir_watch(fd, dir, e->d_name);
 		}
 	}
 	else
@@ -269,7 +272,6 @@ int add_dir_watch(int fd, char * path, char * filename)
 	}
 	closedir(ds);
 	i++;
-	free(buf);
 
 	return(i);
 }
@@ -277,16 +279,15 @@ int add_dir_watch(int fd, char * path, char * filename)
 int
 inotify_insert_file(char * name, const char * path)
 {
-	char * sql;
-	char **result;
-	int rows;
-	char * last_dir = strdup(path);
-	char * path_buf = strdup(path);
-	char * base_name = malloc(strlen(path));
-	char * base_copy = base_name;
+	int len;
+	char * last_dir;
+	char * path_buf;
+	char * base_name;
+	char * base_copy;
 	char * parent_buf = NULL;
 	char * id = NULL;
 	int depth = 1;
+	int ts;
 	enum media_types type = ALL_MEDIA;
 	struct media_dir_s * media_path = media_dirs;
 	struct stat st;
@@ -338,74 +339,59 @@ inotify_insert_file(char * name, const char * path)
 	if( stat(path, &st) != 0 )
 		return -1;
 
-	sql = sqlite3_mprintf("SELECT TIMESTAMP from DETAILS where PATH = '%q'", path);
-	if( sql_get_table(db, sql, &result, &rows, NULL) == SQLITE_OK )
+	ts = sql_get_int_field(db, "SELECT TIMESTAMP from DETAILS where PATH = '%q'", path);
+	if( !ts && is_playlist(path) && (sql_get_int_field(db, "SELECT ID from PLAYLISTS where PATH = '%q'", path) > 0) )
 	{
- 		if( rows )
-		{
-			if( atoi(result[1]) < st.st_mtime )
-			{
-				DPRINTF(E_DEBUG, L_INOTIFY, "%s is newer than the last db entry.\n", path);
-				inotify_remove_file(path_buf);
-			}
-			else
-			{
-				free(last_dir);
-				free(path_buf);
-				free(base_name);
-				sqlite3_free(sql);
-				sqlite3_free_table(result);
-				return -1;
-			}
-		}
-		else if( is_playlist(path) && (sql_get_int_field(db, "SELECT ID from PLAYLISTS where PATH = '%q'", path) > 0) )
-		{
-			DPRINTF(E_DEBUG, L_INOTIFY, "Re-reading modified playlist.\n", path);
-			inotify_remove_file(path_buf);
-			next_pl_fill = 1;
-		}
-		sqlite3_free_table(result);
+		DPRINTF(E_DEBUG, L_INOTIFY, "Re-reading modified playlist.\n", path);
+		inotify_remove_file(path);
+		next_pl_fill = 1;
 	}
-	sqlite3_free(sql);
+	else if( ts < st.st_mtime )
+	{
+		if( ts > 0 )
+			DPRINTF(E_DEBUG, L_INOTIFY, "%s is newer than the last db entry.\n", path);
+		inotify_remove_file(path);
+	}
+
 	/* Find the parentID.  If it's not found, create all necessary parents. */
+	len = strlen(path)+1;
+	if( !(path_buf = malloc(len)) ||
+	    !(last_dir = malloc(len)) ||
+	    !(base_name = malloc(len)) )
+		return -1;
+	base_copy = base_name;
 	while( depth )
 	{
 		depth = 0;
 		strcpy(path_buf, path);
 		parent_buf = dirname(path_buf);
-		strcpy(last_dir, path_buf);
 
 		do
 		{
 			//DEBUG DPRINTF(E_DEBUG, L_INOTIFY, "Checking %s\n", parent_buf);
-			sql = sqlite3_mprintf("SELECT OBJECT_ID from OBJECTS o left join DETAILS d on (d.ID = o.DETAIL_ID)"
-			                      " where d.PATH = '%q' and REF_ID is NULL", parent_buf);
-			if( (sql_get_table(db, sql, &result, &rows, NULL) == SQLITE_OK) && rows )
+			id = sql_get_text_field(db, "SELECT OBJECT_ID from OBJECTS o left join DETAILS d on (d.ID = o.DETAIL_ID)"
+			                            " where d.PATH = '%q' and REF_ID is NULL", parent_buf);
+			if( id )
 			{
-				id = strdup(result[1]);
-				sqlite3_free_table(result);
-				sqlite3_free(sql);
 				if( !depth )
 					break;
-				DPRINTF(E_DEBUG, L_INOTIFY, "Found first known parentID: %s\n", id);
+				DPRINTF(E_DEBUG, L_INOTIFY, "Found first known parentID: %s [%s]\n", id, parent_buf);
 				/* Insert newly-found directory */
 				strcpy(base_name, last_dir);
 				base_copy = basename(base_name);
 				insert_directory(base_copy, last_dir, BROWSEDIR_ID, id+2, get_next_available_id("OBJECTS", id));
-				free(id);
+				sqlite3_free(id);
 				break;
 			}
 			depth++;
-			strcpy(last_dir, path_buf);
+			strcpy(last_dir, parent_buf);
 			parent_buf = dirname(parent_buf);
-			sqlite3_free_table(result);
-			sqlite3_free(sql);
 		}
 		while( strcmp(parent_buf, "/") != 0 );
 
 		if( strcmp(parent_buf, "/") == 0 )
 		{
-			id = strdup(BROWSEDIR_ID);
+			id = sqlite3_mprintf("%s", BROWSEDIR_ID);
 			depth = 0;
 			break;
 		}
@@ -419,7 +405,7 @@ inotify_insert_file(char * name, const char * path)
 	{
 		//DEBUG DPRINTF(E_DEBUG, L_INOTIFY, "Inserting %s\n", name);
 		insert_file(name, path, id+2, get_next_available_id("OBJECTS", id));
-		free(id);
+		sqlite3_free(id);
 		if( (is_audio(path) || is_playlist(path)) && next_pl_fill != 1 )
 		{
 			next_pl_fill = time(NULL) + 120; // Schedule a playlist scan for 2 minutes from now.
@@ -434,32 +420,38 @@ inotify_insert_directory(int fd, char *name, const char * path)
 {
 	DIR * ds;
 	struct dirent * e;
-	char * sql;
-	char **result;
-	char *id=NULL, *path_buf, *parent_buf, *esc_name;
+	char *id, *parent_buf, *esc_name;
+	char path_buf[PATH_MAX];
 	int wd;
-	int rows;
 	enum file_types type = TYPE_UNKNOWN;
 	enum media_types dir_type = ALL_MEDIA;
 	struct media_dir_s * media_path;
 	struct stat st;
 
- 	parent_buf = dirname(strdup(path));
-	sql = sqlite3_mprintf("SELECT OBJECT_ID from OBJECTS o left join DETAILS d on (d.ID = o.DETAIL_ID)"
-	                      " where d.PATH = '%q' and REF_ID is NULL", parent_buf);
-	if( sql_get_table(db, sql, &result, &rows, NULL) == SQLITE_OK )
+	if( access(path, R_OK|X_OK) != 0 )
 	{
-		id = strdup(rows?result[1]:BROWSEDIR_ID);
-		insert_directory(name, path, BROWSEDIR_ID, id+2, get_next_available_id("OBJECTS", id));
-		free(id);
+		DPRINTF(E_WARN, L_INOTIFY, "Could not access %s [%s]\n", path, strerror(errno));
+		return -1;
 	}
+	if( sql_get_int_field(db, "SELECT ID from DETAILS where PATH = '%q'", path) > 0 )
+	{
+		DPRINTF(E_DEBUG, L_INOTIFY, "%s already exists\n", path);
+		return 0;
+	}
+
+ 	parent_buf = strdup(path);
+	id = sql_get_text_field(db, "SELECT OBJECT_ID from OBJECTS o left join DETAILS d on (d.ID = o.DETAIL_ID)"
+	                            " where d.PATH = '%q' and REF_ID is NULL", dirname(parent_buf));
+	if( !id )
+		id = sqlite3_mprintf("%s", BROWSEDIR_ID);
+	insert_directory(name, path, BROWSEDIR_ID, id+2, get_next_available_id("OBJECTS", id));
+	sqlite3_free(id);
 	free(parent_buf);
-	sqlite3_free(sql);
 
 	wd = add_watch(fd, path);
 	if( wd == -1 )
 	{
-		DPRINTF(E_ERROR, L_INOTIFY, "add_watch() failed");
+		DPRINTF(E_ERROR, L_INOTIFY, "add_watch() failed\n");
 	}
 	else
 	{
@@ -487,10 +479,8 @@ inotify_insert_directory(int fd, char *name, const char * path)
 	{
 		if( e->d_name[0] == '.' )
 			continue;
-		esc_name = escape_tag(e->d_name);
-		if( !esc_name )
-			esc_name = strdup(e->d_name);
-		asprintf(&path_buf, "%s/%s", path, e->d_name);
+		esc_name = escape_tag(e->d_name, 1);
+		snprintf(path_buf, sizeof(path_buf), "%s/%s", path, e->d_name);
 		switch( e->d_type )
 		{
 			case DT_DIR:
@@ -513,7 +503,6 @@ inotify_insert_directory(int fd, char *name, const char * path)
 			}
 		}
 		free(esc_name);
-		free(path_buf);
 	}
 	closedir(ds);
 
@@ -523,41 +512,43 @@ inotify_insert_directory(int fd, char *name, const char * path)
 int
 inotify_remove_file(const char * path)
 {
-	char * sql;
+	char sql[128];
+	char art_cache[PATH_MAX];
+	char *id;
+	char *ptr;
 	char **result;
-	char * art_cache;
-	sqlite_int64 detailID = 0;
-	int i, rows, children, playlist, ret = 1;
+	sqlite_int64 detailID;
+	int rows, playlist;
 
+	if( ends_with(path, ".srt") )
+	{
+		rows = sql_exec(db, "DELETE from CAPTIONS where PATH = '%q'", path);
+		return rows;
+	}
 	/* Invalidate the scanner cache so we don't insert files into non-existent containers */
 	valid_cache = 0;
 	playlist = is_playlist(path);
-	sql = sqlite3_mprintf("SELECT ID from %s where PATH = '%q'", playlist?"PLAYLISTS":"DETAILS", path);
-	if( (sql_get_table(db, sql, &result, &rows, NULL) == SQLITE_OK) )
-	{
-		if( rows )
-		{
-			detailID = strtoll(result[1], NULL, 10);
-			ret = 0;
-		}
-		sqlite3_free_table(result);
-	}
-	sqlite3_free(sql);
-	if( playlist && detailID )
+	id = sql_get_text_field(db, "SELECT ID from %s where PATH = '%q'", playlist?"PLAYLISTS":"DETAILS", path);
+	if( !id )
+		return 1;
+	detailID = strtoll(id, NULL, 10);
+	sqlite3_free(id);
+	if( playlist )
 	{
 		sql_exec(db, "DELETE from PLAYLISTS where ID = %lld", detailID);
 		sql_exec(db, "DELETE from DETAILS where ID ="
-		             " (SELECT DETAIL_ID from OBJECTS where OBJECT_ID = '%s$%lld')",
+		             " (SELECT DETAIL_ID from OBJECTS where OBJECT_ID = '%s$%llX')",
 		         MUSIC_PLIST_ID, detailID);
-		sql_exec(db, "DELETE from OBJECTS where OBJECT_ID = '%s$%lld' or PARENT_ID = '%s$%lld'",
+		sql_exec(db, "DELETE from OBJECTS where OBJECT_ID = '%s$%llX' or PARENT_ID = '%s$%llX'",
 		         MUSIC_PLIST_ID, detailID, MUSIC_PLIST_ID, detailID);
 	}
-	else if( detailID )
+	else
 	{
 		/* Delete the parent containers if we are about to empty them. */
-		asprintf(&sql, "SELECT PARENT_ID from OBJECTS where DETAIL_ID = %lld", detailID);
+		snprintf(sql, sizeof(sql), "SELECT PARENT_ID from OBJECTS where DETAIL_ID = %lld", detailID);
 		if( (sql_get_table(db, sql, &result, &rows, NULL) == SQLITE_OK) )
 		{
+			int i, children;
 			for( i=1; i <= rows; i++ )
 			{
 				/* If it's a playlist item, adjust the item count of the playlist */
@@ -576,7 +567,9 @@ inotify_remove_file(const char * path)
 					             " (SELECT DETAIL_ID from OBJECTS where OBJECT_ID = '%s')", result[i]);
 					sql_exec(db, "DELETE from OBJECTS where OBJECT_ID = '%s'", result[i]);
 
-					*rindex(result[i], '$') = '\0';
+					ptr = strrchr(result[i], '$');
+					if( ptr )
+						*ptr = '\0';
 					if( sql_get_int_field(db, "SELECT count(*) from OBJECTS where PARENT_ID = '%s'", result[i]) == 0 )
 					{
 						sql_exec(db, "DELETE from DETAILS where ID ="
@@ -587,16 +580,14 @@ inotify_remove_file(const char * path)
 			}
 			sqlite3_free_table(result);
 		}
-		free(sql);
 		/* Now delete the actual objects */
 		sql_exec(db, "DELETE from DETAILS where ID = %lld", detailID);
 		sql_exec(db, "DELETE from OBJECTS where DETAIL_ID = %lld", detailID);
 	}
-	asprintf(&art_cache, "%s/art_cache%s", db_path, path);
+	snprintf(art_cache, sizeof(art_cache), "%s/art_cache%s", db_path, path);
 	remove(art_cache);
-	free(art_cache);
 
-	return ret;
+	return 0;
 }
 
 int
@@ -607,6 +598,8 @@ inotify_remove_directory(int fd, const char * path)
 	sqlite_int64 detailID = 0;
 	int rows, i, ret = 1;
 
+	/* Invalidate the scanner cache so we don't insert files into non-existent containers */
+	valid_cache = 0;
 	remove_watch(fd, path);
 	sql = sqlite3_mprintf("SELECT ID from DETAILS where PATH glob '%q/*'"
 	                      " UNION ALL SELECT ID from DETAILS where PATH = '%q'", path, path);
@@ -645,9 +638,8 @@ start_inotify()
 	pollfds[0].fd = inotify_init();
 	pollfds[0].events = POLLIN;
 
-	if ( pollfds[0].fd < 0 ) {
+	if ( pollfds[0].fd < 0 )
 		DPRINTF(E_ERROR, L_INOTIFY, "inotify_init() failed!\n");
-	}
 
 	while( scanning )
 	{
@@ -658,6 +650,7 @@ start_inotify()
 	inotify_create_watches(pollfds[0].fd);
 	if (setpriority(PRIO_PROCESS, 0, 19) == -1)
 		DPRINTF(E_WARN, L_INOTIFY,  "Failed to reduce inotify thread priority\n");
+	sqlite3_release_memory(1<<31);
         
 	while( !quitting )
 	{
@@ -696,32 +689,44 @@ start_inotify()
 				}
 				esc_name = modifyString(strdup(event->name), "&", "&amp;amp;", 0);
 				sprintf(path_buf, "%s/%s", get_path_from_wd(event->wd), event->name);
-				if ( (event->mask & IN_CREATE && event->mask & IN_ISDIR) ||
-				     (event->mask & IN_MOVED_TO && event->mask & IN_ISDIR) )
+				if ( event->mask & IN_ISDIR && (event->mask & (IN_CREATE|IN_MOVED_TO)) )
 				{
-					DPRINTF(E_DEBUG, L_INOTIFY,  "The directory %s was %s.\n", path_buf, (event->mask & IN_MOVED_TO ? "moved here" : "created"));
+					DPRINTF(E_DEBUG, L_INOTIFY,  "The directory %s was %s.\n",
+						path_buf, (event->mask & IN_MOVED_TO ? "moved here" : "created"));
 					inotify_insert_directory(pollfds[0].fd, esc_name, path_buf);
 				}
-				else if ( event->mask & IN_CLOSE_WRITE || event->mask & IN_MOVED_TO )
+				else if ( (event->mask & (IN_CLOSE_WRITE|IN_MOVED_TO|IN_CREATE)) &&
+				          (lstat(path_buf, &st) == 0) )
 				{
-					if( stat(path_buf, &st) == 0 && st.st_size > 0 )
+					if( S_ISLNK(st.st_mode) )
 					{
-						DPRINTF(E_DEBUG, L_INOTIFY, "The file %s was %s.\n", path_buf, (event->mask & IN_MOVED_TO ? "moved here" : "changed"));
-						inotify_insert_file(esc_name, path_buf);
+						DPRINTF(E_DEBUG, L_INOTIFY, "The symbolic link %s was %s.\n",
+							path_buf, (event->mask & IN_MOVED_TO ? "moved here" : "created"));
+						if( stat(path_buf, &st) == 0 && S_ISDIR(st.st_mode) )
+							inotify_insert_directory(pollfds[0].fd, esc_name, path_buf);
+						else
+							inotify_insert_file(esc_name, path_buf);
+					}
+					else if( event->mask & (IN_CLOSE_WRITE|IN_MOVED_TO) && st.st_size > 0 )
+					{
+						if( (event->mask & IN_MOVED_TO) ||
+						    (sql_get_int_field(db, "SELECT TIMESTAMP from DETAILS where PATH = '%q'", path_buf) != st.st_mtime) )
+						{
+							DPRINTF(E_DEBUG, L_INOTIFY, "The file %s was %s.\n",
+								path_buf, (event->mask & IN_MOVED_TO ? "moved here" : "changed"));
+							inotify_insert_file(esc_name, path_buf);
+						}
 					}
 				}
-				else if ( event->mask & IN_DELETE || event->mask & IN_MOVED_FROM )
+				else if ( event->mask & (IN_DELETE|IN_MOVED_FROM) )
 				{
+					DPRINTF(E_DEBUG, L_INOTIFY, "The %s %s was %s.\n",
+						(event->mask & IN_ISDIR ? "directory" : "file"),
+						path_buf, (event->mask & IN_MOVED_FROM ? "moved away" : "deleted"));
 					if ( event->mask & IN_ISDIR )
-					{
-						DPRINTF(E_DEBUG, L_INOTIFY, "The directory %s was %s.\n", path_buf, (event->mask & IN_MOVED_FROM ? "moved away" : "deleted"));
 						inotify_remove_directory(pollfds[0].fd, path_buf);
-					}
 					else
-					{
-						DPRINTF(E_DEBUG, L_INOTIFY, "The file %s was %s.\n", path_buf, (event->mask & IN_MOVED_FROM ? "moved away" : "deleted"));
 						inotify_remove_file(path_buf);
-					}
 				}
 				free(esc_name);
 			}
