@@ -52,6 +52,12 @@
 #ifndef FF_PROFILE_H264_HIGH
 #define FF_PROFILE_H264_HIGH 100
 #endif
+#define FF_PROFILE_SKIP -100
+
+#if LIBAVCODEC_VERSION_MAJOR < 53
+#define AVMEDIA_TYPE_AUDIO CODEC_TYPE_AUDIO
+#define AVMEDIA_TYPE_VIDEO CODEC_TYPE_VIDEO
+#endif
 
 #define FLAG_TITLE	0x00000001
 #define FLAG_ARTIST	0x00000002
@@ -68,6 +74,7 @@
 #define FLAG_FREQUENCY	0x00001000
 #define FLAG_BPS	0x00002000
 #define FLAG_CHANNELS	0x00004000
+#define FLAG_ROTATION	0x00008000
 
 /* Audio profile flags */
 enum audio_profiles {
@@ -86,34 +93,41 @@ enum audio_profiles {
 
 /* This function shamelessly copied from libdlna */
 #define MPEG_TS_SYNC_CODE 0x47
-#define MPEG_TS_PACKET_LENGTH 188 /* prepends 4 bytes to TS packet */
+#define MPEG_TS_PACKET_LENGTH 188
 #define MPEG_TS_PACKET_LENGTH_DLNA 192 /* prepends 4 bytes to TS packet */
 int
-dlna_timestamp_is_present(const char * filename)
+dlna_timestamp_is_present(const char * filename, int * raw_packet_size)
 {
-	unsigned char buffer[2*MPEG_TS_PACKET_LENGTH_DLNA+1];
+	unsigned char buffer[3*MPEG_TS_PACKET_LENGTH_DLNA];
 	int fd, i;
 
 	/* read file header */
 	fd = open(filename, O_RDONLY);
-	read(fd, buffer, MPEG_TS_PACKET_LENGTH_DLNA*2);
+	read(fd, buffer, MPEG_TS_PACKET_LENGTH_DLNA*3);
 	close(fd);
 	for( i=0; i < MPEG_TS_PACKET_LENGTH_DLNA; i++ )
 	{
 		if( buffer[i] == MPEG_TS_SYNC_CODE )
 		{
-			if (buffer[i + MPEG_TS_PACKET_LENGTH_DLNA] == MPEG_TS_SYNC_CODE)
+			if (buffer[i + MPEG_TS_PACKET_LENGTH_DLNA] == MPEG_TS_SYNC_CODE &&
+			    buffer[i + MPEG_TS_PACKET_LENGTH_DLNA*2] == MPEG_TS_SYNC_CODE)
 			{
+			        *raw_packet_size = MPEG_TS_PACKET_LENGTH_DLNA;
 				if (buffer[i+MPEG_TS_PACKET_LENGTH] == 0x00 &&
 				    buffer[i+MPEG_TS_PACKET_LENGTH+1] == 0x00 &&
 				    buffer[i+MPEG_TS_PACKET_LENGTH+2] == 0x00 &&
 				    buffer[i+MPEG_TS_PACKET_LENGTH+3] == 0x00)
-					break;
+					return 0;
 				else
 					return 1;
+			} else if (buffer[i + MPEG_TS_PACKET_LENGTH] == MPEG_TS_SYNC_CODE &&
+				   buffer[i + MPEG_TS_PACKET_LENGTH*2] == MPEG_TS_SYNC_CODE) {
+			    *raw_packet_size = MPEG_TS_PACKET_LENGTH;
+			    return 0;
 			}
 		}
 	}
+	*raw_packet_size = 0;
 	return 0;
 }
 
@@ -228,40 +242,42 @@ parse_nfo(const char * path, metadata_t * m)
 void
 free_metadata(metadata_t * m, uint32_t flags)
 {
-	if( m->title && (flags & FLAG_TITLE) )
+	if( flags & FLAG_TITLE )
 		free(m->title);
-	if( m->artist && (flags & FLAG_ARTIST) )
+	if( flags & FLAG_ARTIST )
 		free(m->artist);
-	if( m->album && (flags & FLAG_ALBUM) )
+	if( flags & FLAG_ALBUM )
 		free(m->album);
-	if( m->genre && (flags & FLAG_GENRE) )
+	if( flags & FLAG_GENRE )
 		free(m->genre);
-	if( m->creator && (flags & FLAG_CREATOR) )
+	if( flags & FLAG_CREATOR )
 		free(m->creator);
-	if( m->date && (flags & FLAG_DATE) )
+	if( flags & FLAG_DATE )
 		free(m->date);
-	if( m->comment && (flags & FLAG_COMMENT) )
+	if( flags & FLAG_COMMENT )
 		free(m->comment);
-	if( m->dlna_pn && (flags & FLAG_DLNA_PN) )
+	if( flags & FLAG_DLNA_PN )
 		free(m->dlna_pn);
-	if( m->mime && (flags & FLAG_MIME) )
+	if( flags & FLAG_MIME )
 		free(m->mime);
-	if( m->duration && (flags & FLAG_DURATION) )
+	if( flags & FLAG_DURATION )
 		free(m->duration);
-	if( m->resolution && (flags & FLAG_RESOLUTION) )
+	if( flags & FLAG_RESOLUTION )
 		free(m->resolution);
-	if( m->bitrate && (flags & FLAG_BITRATE) )
+	if( flags & FLAG_BITRATE )
 		free(m->bitrate);
-	if( m->frequency && (flags & FLAG_FREQUENCY) )
+	if( flags & FLAG_FREQUENCY )
 		free(m->frequency);
-	if( m->bps && (flags & FLAG_BPS) )
+	if( flags & FLAG_BPS )
 		free(m->bps);
-	if( m->channels && (flags & FLAG_CHANNELS) )
+	if( flags & FLAG_CHANNELS )
 		free(m->channels);
+	if( flags & FLAG_ROTATION )
+		free(m->rotation);
 }
 
 sqlite_int64
-GetFolderMetadata(const char * name, const char * path, const char * artist, const char * genre, const char * album_art)
+GetFolderMetadata(const char * name, const char * path, const char * artist, const char * genre, sqlite3_int64 album_art)
 {
 	int ret;
 
@@ -269,8 +285,7 @@ GetFolderMetadata(const char * name, const char * path, const char * artist, con
 	                   " (TITLE, PATH, CREATOR, ARTIST, GENRE, ALBUM_ART) "
 	                   "VALUES"
 	                   " ('%q', %Q, %Q, %Q, %Q, %lld);",
-	                   name, path, artist, artist, genre,
-	                   album_art ? strtoll(album_art, NULL, 10) : 0);
+	                   name, path, artist, artist, genre, album_art);
 	if( ret != SQLITE_OK )
 		ret = 0;
 	else
@@ -350,8 +365,7 @@ GetAudioMetadata(const char * path, char * name)
 		if( !getenv("LANG") )
 			strcpy(lang, "en_US");
 		else
-			strncpy(lang, getenv("LANG"), 5);
-		lang[5] = '\0';
+			strncpyt(lang, getenv("LANG"), sizeof(lang));
 	}
 
 	if( readtags((char *)path, &song, &file, lang, type) != 0 )
@@ -363,7 +377,7 @@ GetAudioMetadata(const char * path, char * name)
 	}
 
 	if( song.dlna_pn )
-		asprintf(&m.dlna_pn, "%s;DLNA.ORG_OP=01;DLNA.ORG_CI=0", song.dlna_pn);
+		m.dlna_pn = strdup(song.dlna_pn);
 	if( song.year )
 		asprintf(&m.date, "%04d-01-01", song.year);
 	asprintf(&m.duration, "%d:%02d:%02d.%03d",
@@ -457,7 +471,7 @@ GetAudioMetadata(const char * path, char * name)
 	                   "  TITLE, CREATOR, ARTIST, ALBUM, GENRE, COMMENT, DISC, TRACK, DLNA_PN, MIME, ALBUM_ART) "
 	                   "VALUES"
 	                   " (%Q, %lld, %ld, '%s', %d, %d, %d, %Q, %Q, %Q, %Q, %Q, %Q, %Q, %d, %d, %Q, '%s', %lld);",
-	                   path, file.st_size, file.st_mtime, m.duration, song.channels, song.bitrate, song.samplerate, m.date,
+	                   path, (long long)file.st_size, file.st_mtime, m.duration, song.channels, song.bitrate, song.samplerate, m.date,
 	                   m.title, m.creator, m.artist, m.album, m.genre, m.comment, song.disc, song.track,
 	                   m.dlna_pn, song.mime?song.mime:m.mime, album_art);
 	if( ret != SQLITE_OK )
@@ -511,7 +525,7 @@ GetImageMetadata(const char * path, char * name)
 	//DEBUG DPRINTF(E_DEBUG, L_METADATA, " * size: %jd\n", file.st_size);
 
 	/* MIME hard-coded to JPEG for now, until we add PNG support */
-	asprintf(&m.mime, "image/jpeg");
+	m.mime = strdup("image/jpeg");
 
 	l = exif_loader_new();
 	exif_loader_write_file(l, path);
@@ -521,7 +535,8 @@ GetImageMetadata(const char * path, char * name)
 		goto no_exifdata;
 
 	e = exif_content_get_entry (ed->ifd[EXIF_IFD_EXIF], EXIF_TAG_DATE_TIME_ORIGINAL);
-	if( e || (e = exif_content_get_entry(ed->ifd[EXIF_IFD_EXIF], EXIF_TAG_DATE_TIME_DIGITIZED)) ) {
+	if( e || (e = exif_content_get_entry(ed->ifd[EXIF_IFD_EXIF], EXIF_TAG_DATE_TIME_DIGITIZED)) )
+	{
 		m.date = strdup(exif_entry_get_value(e, b, sizeof(b)));
 		if( strlen(m.date) > 10 )
 		{
@@ -540,27 +555,53 @@ GetImageMetadata(const char * path, char * name)
 	}
 	//DEBUG DPRINTF(E_DEBUG, L_METADATA, " * date: %s\n", m.date);
 
-	e = exif_content_get_entry (ed->ifd[EXIF_IFD_0], EXIF_TAG_MAKE);
+	e = exif_content_get_entry(ed->ifd[EXIF_IFD_0], EXIF_TAG_MAKE);
 	if( e )
 	{
-		strncpy(make, exif_entry_get_value(e, b, sizeof(b)), sizeof(make));
-		e = exif_content_get_entry (ed->ifd[EXIF_IFD_0], EXIF_TAG_MODEL);
+		strncpyt(make, exif_entry_get_value(e, b, sizeof(b)), sizeof(make));
+		e = exif_content_get_entry(ed->ifd[EXIF_IFD_0], EXIF_TAG_MODEL);
 		if( e )
 		{
-			strncpy(model, exif_entry_get_value(e, b, sizeof(b)), sizeof(model));
+			strncpyt(model, exif_entry_get_value(e, b, sizeof(b)), sizeof(model));
 			if( !strcasestr(model, make) )
 				snprintf(model, sizeof(model), "%s %s", make, exif_entry_get_value(e, b, sizeof(b)));
-			m.creator = strdup(model);
+			m.creator = escape_tag(trim(model), 1);
 		}
 	}
 	//DEBUG DPRINTF(E_DEBUG, L_METADATA, " * model: %s\n", model);
 
+	e = exif_content_get_entry(ed->ifd[EXIF_IFD_0], EXIF_TAG_ORIENTATION);
+	if( e )
+	{
+		int rotate;
+		switch( exif_get_short(e->data, exif_data_get_byte_order(ed)) )
+		{
+		case 3:
+			rotate = 180;
+			break;
+		case 6:
+			rotate = 90;
+			break;
+		case 8:
+			rotate = 270;
+			break;
+		default:
+			rotate = 0;
+			break;
+		}
+		if( rotate )
+		{
+			if( asprintf(&m.rotation, "%d", rotate) < 0 )
+				m.rotation = NULL;
+		}
+	}
+
 	if( ed->size )
 	{
 		/* We might need to verify that the thumbnail is 160x160 or smaller */
 		if( ed->size > 12000 )
 		{
-			imsrc = image_new_from_jpeg(NULL, 0, (char *)ed->data, ed->size, 1);
+			imsrc = image_new_from_jpeg(NULL, 0, (char *)ed->data, ed->size, 1, ROTATE_NONE);
 			if( imsrc )
 			{
  				if( (imsrc->width <= 160) && (imsrc->height <= 160) )
@@ -580,19 +621,22 @@ no_exifdata:
 	if( image_get_jpeg_resolution(path, &width, &height) != 0 || !width || !height )
 	{
 		infile = fopen(path, "r");
-		cinfo.err = jpeg_std_error(&jerr);
-		jerr.error_exit = libjpeg_error_handler;
-		jpeg_create_decompress(&cinfo);
-		if( setjmp(setjmp_buffer) )
-			goto error;
-		jpeg_stdio_src(&cinfo, infile);
-		jpeg_read_header(&cinfo, TRUE);
-		jpeg_start_decompress(&cinfo);
-		width = cinfo.output_width;
-		height = cinfo.output_height;
-		error:
-		jpeg_destroy_decompress(&cinfo);
-		fclose(infile);
+		if( infile )
+		{
+			cinfo.err = jpeg_std_error(&jerr);
+			jerr.error_exit = libjpeg_error_handler;
+			jpeg_create_decompress(&cinfo);
+			if( setjmp(setjmp_buffer) )
+				goto error;
+			jpeg_stdio_src(&cinfo, infile);
+			jpeg_read_header(&cinfo, TRUE);
+			jpeg_start_decompress(&cinfo);
+			width = cinfo.output_width;
+			height = cinfo.output_height;
+			error:
+			jpeg_destroy_decompress(&cinfo);
+			fclose(infile);
+		}
 	}
 	//DEBUG DPRINTF(E_DEBUG, L_METADATA, " * resolution: %dx%d\n", width, height);
 
@@ -602,18 +646,20 @@ no_exifdata:
 		return 0;
 	}
 	if( width <= 640 && height <= 480 )
-		asprintf(&m.dlna_pn, "JPEG_SM;%s", dlna_no_conv);
+		m.dlna_pn = strdup("JPEG_SM");
 	else if( width <= 1024 && height <= 768 )
-		asprintf(&m.dlna_pn, "JPEG_MED;%s", dlna_no_conv);
+		m.dlna_pn = strdup("JPEG_MED");
 	else if( (width <= 4096 && height <= 4096) || !(GETFLAG(DLNA_STRICT_MASK)) )
-		asprintf(&m.dlna_pn, "JPEG_LRG;%s", dlna_no_conv);
+		m.dlna_pn = strdup("JPEG_LRG");
 	asprintf(&m.resolution, "%dx%d", width, height);
 
 	ret = sql_exec(db, "INSERT into DETAILS"
-	                   " (PATH, TITLE, SIZE, TIMESTAMP, DATE, RESOLUTION, THUMBNAIL, CREATOR, DLNA_PN, MIME) "
+	                   " (PATH, TITLE, SIZE, TIMESTAMP, DATE, RESOLUTION,"
+	                    " ROTATION, THUMBNAIL, CREATOR, DLNA_PN, MIME) "
 	                   "VALUES"
-	                   " (%Q, '%q', %lld, %ld, %Q, %Q, %d, %Q, %Q, %Q);",
-	                   path, name, file.st_size, file.st_mtime, m.date, m.resolution, thumb, m.creator, m.dlna_pn, m.mime);
+	                   " (%Q, '%q', %lld, %ld, %Q, %Q, %Q, %d, %Q, %Q, %Q);",
+	                   path, name, (long long)file.st_size, file.st_mtime, m.date, m.resolution,
+	                   m.rotation, thumb, m.creator, m.dlna_pn, m.mime);
 	if( ret != SQLITE_OK )
 	{
 		fprintf(stderr, "Error inserting details for '%s'!\n", path);
@@ -634,17 +680,17 @@ GetVideoMetadata(const char * path, char * name)
 	struct stat file;
 	int ret, i;
 	struct tm *modtime;
-	AVFormatContext *ctx;
+	AVFormatContext *ctx = NULL;
 	AVCodecContext *ac = NULL, *vc = NULL;
 	int audio_stream = -1, video_stream = -1;
 	enum audio_profiles audio_profile = PROFILE_AUDIO_UNKNOWN;
-	tsinfo_t *ts;
 	char fourcc[4];
 	sqlite_int64 album_art = 0;
 	char nfo[PATH_MAX], *ext;
 	struct song_metadata video;
 	metadata_t m;
 	uint32_t free_flags = 0xFFFFFFFF;
+
 	memset(&m, '\0', sizeof(m));
 	memset(&video, '\0', sizeof(video));
 
@@ -655,7 +701,11 @@ GetVideoMetadata(const char * path, char * name)
 	//DEBUG DPRINTF(E_DEBUG, L_METADATA, " * size: %jd\n", file.st_size);
 
 	av_register_all();
+	#if LIBAVFORMAT_VERSION_INT >= ((53<<16)+(2<<8)+0)
+	if( avformat_open_input(&ctx, path, NULL, NULL) != 0 )
+	#else
 	if( av_open_input_file(&ctx, path, NULL, 0, NULL) != 0 )
+	#endif
 	{
 		DPRINTF(E_WARN, L_METADATA, "Opening %s failed!\n", path);
 		return 0;
@@ -665,14 +715,14 @@ GetVideoMetadata(const char * path, char * name)
 	for( i=0; i<ctx->nb_streams; i++)
 	{
 		if( audio_stream == -1 &&
-		    ctx->streams[i]->codec->codec_type == CODEC_TYPE_AUDIO )
+		    ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO )
 		{
 			audio_stream = i;
 			ac = ctx->streams[audio_stream]->codec;
 			continue;
 		}
 		else if( video_stream == -1 &&
-			 ctx->streams[i]->codec->codec_type == CODEC_TYPE_VIDEO )
+			 ctx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO )
 		{
 			video_stream = i;
 			vc = ctx->streams[video_stream]->codec;
@@ -848,7 +898,7 @@ GetVideoMetadata(const char * path, char * name)
 					if( (vc->width  == 352) &&
 					    (vc->height <= 288) )
 					{
-						asprintf(&m.dlna_pn, "MPEG1;%s", dlna_no_conv);
+						m.dlna_pn = strdup("MPEG1");
 					}
 					asprintf(&m.mime, "video/mpeg");
 				}
@@ -858,8 +908,10 @@ GetVideoMetadata(const char * path, char * name)
 				off = sprintf(m.dlna_pn, "MPEG_");
 				if( strcmp(ctx->iformat->name, "mpegts") == 0 )
 				{
-					DPRINTF(E_DEBUG, L_METADATA, "Stream %d of %s is %s MPEG2 TS\n",
-						video_stream, basename(path), m.resolution);
+					int raw_packet_size;
+					int dlna_ts_present = dlna_timestamp_is_present(path, &raw_packet_size);
+					DPRINTF(E_DEBUG, L_METADATA, "Stream %d of %s is %s MPEG2 TS packet size %d\n",
+						video_stream, basename(path), m.resolution, raw_packet_size);
 					off += sprintf(m.dlna_pn+off, "TS_");
 					if( (vc->width  >= 1280) &&
 					    (vc->height >= 720) )
@@ -875,18 +927,17 @@ GetVideoMetadata(const char * path, char * name)
 						else
 							off += sprintf(m.dlna_pn+off, "NA");
 					}
-					ts = ctx->priv_data;
-					if( ts->packet_size == MPEG_TS_PACKET_LENGTH_DLNA )
+					if( raw_packet_size == MPEG_TS_PACKET_LENGTH_DLNA )
 					{
-						if( dlna_timestamp_is_present(path) )
+						if (dlna_ts_present)
 							ts_timestamp = VALID;
 						else
 							ts_timestamp = EMPTY;
 					}
-					else if( ts->packet_size != MPEG_TS_PACKET_LENGTH )
+					else if( raw_packet_size != MPEG_TS_PACKET_LENGTH )
 					{
 						DPRINTF(E_DEBUG, L_METADATA, "Unsupported DLNA TS packet size [%d] (%s)\n",
-							ts->packet_size, basename(path));
+							raw_packet_size, basename(path));
 						free(m.dlna_pn);
 						m.dlna_pn = NULL;
 					}
@@ -924,8 +975,6 @@ GetVideoMetadata(const char * path, char * name)
 					free(m.dlna_pn);
 					m.dlna_pn = NULL;
 				}
-				if( m.dlna_pn )
-					sprintf(m.dlna_pn+off, ";%s", dlna_no_conv);
 				break;
 			case CODEC_ID_H264:
 				m.dlna_pn = malloc(128);
@@ -933,7 +982,44 @@ GetVideoMetadata(const char * path, char * name)
 
 				if( strcmp(ctx->iformat->name, "mpegts") == 0 )
 				{
+					AVRational display_aspect_ratio;
+					int fps, interlaced;
+					int raw_packet_size;
+					int dlna_ts_present = dlna_timestamp_is_present(path, &raw_packet_size);
+
 					off += sprintf(m.dlna_pn+off, "TS_");
+					if (vc->sample_aspect_ratio.num) {
+						av_reduce(&display_aspect_ratio.num, &display_aspect_ratio.den,
+						          vc->width  * vc->sample_aspect_ratio.num,
+						          vc->height * vc->sample_aspect_ratio.den,
+						          1024*1024);
+					}
+					fps = ctx->streams[video_stream]->r_frame_rate.num / ctx->streams[video_stream]->r_frame_rate.den;
+					interlaced = (ctx->streams[video_stream]->r_frame_rate.num / vc->time_base.den);
+					if( ((((vc->width == 1920 || vc->width == 1440) && vc->height == 1080) ||
+					      (vc->width == 720 && vc->height == 480)) && fps == 59 && interlaced) ||
+					    ((vc->width == 1280 && vc->height == 720) && fps == 59 && !interlaced) )
+					{
+						if( (vc->profile == FF_PROFILE_H264_MAIN || vc->profile == FF_PROFILE_H264_HIGH) &&
+						    audio_profile == PROFILE_AUDIO_AC3 )
+						{
+							off += sprintf(m.dlna_pn+off, "HD_60_");
+							vc->profile = FF_PROFILE_SKIP;
+						}
+					}
+					else if( ((vc->width == 1920 && vc->height == 1080) ||
+					          (vc->width == 1440 && vc->height == 1080) ||
+					          (vc->width == 1280 && vc->height ==  720) ||
+					          (vc->width ==  720 && vc->height ==  576)) &&
+					          interlaced && fps == 50 )
+					{
+						if( (vc->profile == FF_PROFILE_H264_MAIN || vc->profile == FF_PROFILE_H264_HIGH) &&
+						    audio_profile == PROFILE_AUDIO_AC3 )
+						{
+							off += sprintf(m.dlna_pn+off, "HD_50_");
+							vc->profile = FF_PROFILE_SKIP;
+						}
+					}
 					switch( vc->profile )
 					{
 						case FF_PROFILE_H264_BASELINE:
@@ -1001,6 +1087,8 @@ GetVideoMetadata(const char * path, char * name)
 								m.dlna_pn = NULL;
 							}
 							break;
+						case FF_PROFILE_SKIP:
+							break;
 					}
 					if( !m.dlna_pn )
 						break;
@@ -1025,19 +1113,18 @@ GetVideoMetadata(const char * path, char * name)
 					}
 					if( !m.dlna_pn )
 						break;
-					ts = ctx->priv_data;
-					if( ts->packet_size == MPEG_TS_PACKET_LENGTH_DLNA )
+					if( raw_packet_size == MPEG_TS_PACKET_LENGTH_DLNA )
 					{
 						if( vc->profile == FF_PROFILE_H264_HIGH ||
-						    dlna_timestamp_is_present(path) )
+						    dlna_ts_present )
 							ts_timestamp = VALID;
 						else
 							ts_timestamp = EMPTY;
 					}
-					else if( ts->packet_size != MPEG_TS_PACKET_LENGTH )
+					else if( raw_packet_size != MPEG_TS_PACKET_LENGTH )
 					{
 						DPRINTF(E_DEBUG, L_METADATA, "Unsupported DLNA TS packet size [%d] (%s)\n",
-							ts->packet_size, basename(path));
+							raw_packet_size, basename(path));
 						free(m.dlna_pn);
 						m.dlna_pn = NULL;
 					}
@@ -1192,8 +1279,6 @@ GetVideoMetadata(const char * path, char * name)
 					free(m.dlna_pn);
 					m.dlna_pn = NULL;
 				}
-				if( m.dlna_pn )
-					sprintf(m.dlna_pn+off, ";%s", dlna_no_conv);
 				DPRINTF(E_DEBUG, L_METADATA, "Stream %d of %s is h.264\n", video_stream, basename(path));
 				break;
 			case CODEC_ID_MPEG4:
@@ -1257,8 +1342,6 @@ GetVideoMetadata(const char * path, char * name)
 							m.dlna_pn = NULL;
 						}
 					}
-					if( m.dlna_pn )
-						sprintf(m.dlna_pn+off, ";%s", dlna_no_conv);
 				}
 				break;
 			case CODEC_ID_WMV3:
@@ -1368,8 +1451,6 @@ GetVideoMetadata(const char * path, char * name)
 							break;
 					}
 				}
-				if( m.dlna_pn )
-					sprintf(m.dlna_pn+off, ";%s", dlna_no_conv);
 				break;
 			case CODEC_ID_MSMPEG4V3:
 				asprintf(&m.mime, "video/x-msvideo");
@@ -1478,7 +1559,7 @@ video_no_dlna:
 	                   "  TITLE, CREATOR, ARTIST, GENRE, COMMENT, DLNA_PN, MIME, ALBUM_ART) "
 	                   "VALUES"
 	                   " (%Q, %lld, %ld, %Q, %Q, %Q, %Q, %Q, %Q, '%q', %Q, %Q, %Q, %Q, %Q, '%q', %lld);",
-	                   path, file.st_size, file.st_mtime, m.duration,
+	                   path, (long long)file.st_size, file.st_mtime, m.duration,
 	                   m.date, m.channels, m.bitrate, m.frequency, m.resolution,
 			   m.title, m.creator, m.artist, m.genre, m.comment, m.dlna_pn,
                            m.mime, album_art);
