@@ -1,19 +1,19 @@
-/*  MiniDLNA media server
- *  Copyright (C) 2008-2009  Justin Maggard
+/* MiniDLNA media server
+ * Copyright (C) 2008-2009  Justin Maggard
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This file is part of MiniDLNA.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * MiniDLNA is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * MiniDLNA is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with MiniDLNA. If not, see <http://www.gnu.org/licenses/>.
  */
 #include <stdio.h>
 #include <string.h>
@@ -22,6 +22,7 @@
 #include <dirent.h>
 #include <locale.h>
 #include <libgen.h>
+#include <inttypes.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -39,13 +40,14 @@
 #include "utils.h"
 #include "sql.h"
 #include "scanner.h"
+#include "albumart.h"
 #include "log.h"
 
 int valid_cache = 0;
 
 struct virtual_item
 {
-	int  objectID;
+	sqlite3_int64 objectID;
 	char parentID[64];
 	char name[256];
 };
@@ -53,87 +55,81 @@ struct virtual_item
 sqlite_int64
 get_next_available_id(const char * table, const char * parentID)
 {
-		char * sql;
-		char **result;
-		int ret, rows;
+		char *ret, *base;
 		sqlite_int64 objectID = 0;
 
-		asprintf(&sql, "SELECT OBJECT_ID from %s where ID = "
-		               "(SELECT max(ID) from %s where PARENT_ID = '%s')", table, table, parentID);
-		ret = sql_get_table(db, sql, &result, &rows, NULL);
-		if( rows )
+		ret = sql_get_text_field(db, "SELECT OBJECT_ID from %s where ID = "
+		                             "(SELECT max(ID) from %s where PARENT_ID = '%s')",
+		                             table, table, parentID);
+		if( ret )
 		{
-			objectID = strtoll(strrchr(result[1], '$')+1, NULL, 16) + 1;
+			base = strrchr(ret, '$');
+			if( base )
+				objectID = strtoll(base+1, NULL, 16) + 1;
+			sqlite3_free(ret);
 		}
-		sqlite3_free_table(result);
-		free(sql);
 
 		return objectID;
 }
 
-long long int
+int
 insert_container(const char * item, const char * rootParent, const char * refID, const char *class,
-                 const char *artist, const char *genre, const char *album_art)
+                 const char *artist, const char *genre, const char *album_art, sqlite3_int64 *objectID, sqlite3_int64 *parentID)
 {
-	char **result;
-	char **result2;
-	char *sql;
-	int cols, rows, ret;
-	int parentID = 0, objectID = 0;
+	char *result;
+	char *base;
+	int ret = 0;
 	sqlite_int64 detailID = 0;
 
-	sql = sqlite3_mprintf("SELECT OBJECT_ID from OBJECTS"
-	                      " where PARENT_ID = '%s'"
-			      " and NAME = '%q'"
-	                      " and CLASS = 'container.%s' limit 1",
-	                      rootParent, item, class);
-	ret = sql_get_table(db, sql, &result, &rows, &cols);
-	sqlite3_free(sql);
-	if( cols )
+	result = sql_get_text_field(db, "SELECT OBJECT_ID from OBJECTS o "
+	                                "left join DETAILS d on (o.DETAIL_ID = d.ID)"
+	                                " where o.PARENT_ID = '%s'"
+			                " and o.NAME like '%q'"
+			                " and d.ARTIST %s %Q"
+	                                " and o.CLASS = 'container.%s' limit 1",
+	                                rootParent, item, artist?"like":"is", artist, class);
+	if( result )
 	{
-		parentID = strtol(rindex(result[1], '$')+1, NULL, 16);
-		objectID = get_next_available_id("OBJECTS", result[1]);
+		base = strrchr(result, '$');
+		if( base )
+			*parentID = strtoll(base+1, NULL, 16);
+		else
+			*parentID = 0;
+		*objectID = get_next_available_id("OBJECTS", result);
 	}
 	else
 	{
-		parentID = get_next_available_id("OBJECTS", rootParent);
+		*objectID = 0;
+		*parentID = get_next_available_id("OBJECTS", rootParent);
 		if( refID )
 		{
-			sql = sqlite3_mprintf("SELECT DETAIL_ID from OBJECTS where OBJECT_ID = %Q", refID);
-			ret = sql_get_table(db, sql, &result2, &rows, NULL);
-			if( ret == SQLITE_OK )
-			{
-				if( rows )
-				{
-					detailID = strtoll(result2[1], NULL, 10);
-				}
-				sqlite3_free_table(result2);
-			}
-			sqlite3_free(sql);
+			result = sql_get_text_field(db, "SELECT DETAIL_ID from OBJECTS where OBJECT_ID = %Q", refID);
+			if( result )
+				detailID = strtoll(result, NULL, 10);
 		}
 		if( !detailID )
 		{
-			detailID = GetFolderMetadata(item, NULL, artist, genre, album_art);
+			detailID = GetFolderMetadata(item, NULL, artist, genre, (album_art ? strtoll(album_art, NULL, 10) : 0));
 		}
 		ret = sql_exec(db, "INSERT into OBJECTS"
 		                   " (OBJECT_ID, PARENT_ID, REF_ID, DETAIL_ID, CLASS, NAME) "
 		                   "VALUES"
-		                   " ('%s$%X', '%s', %Q, %lld, 'container.%s', '%q')",
-		                   rootParent, parentID, rootParent, refID, detailID, class, item);
+		                   " ('%s$%"PRIX64"', '%s', %Q, %"PRId64", 'container.%s', '%q')",
+		                   rootParent, *parentID, rootParent, refID, detailID, class, item);
 	}
-	sqlite3_free_table(result);
+	sqlite3_free(result);
 
-	return (long long)parentID<<32|objectID;
+	return ret;
 }
 
-void
-insert_containers(const char * name, const char *path, const char * refID, const char * class, long unsigned int detailID)
+static void
+insert_containers(const char * name, const char *path, const char * refID, const char * class, sqlite3_int64 detailID)
 {
-	char *sql;
+	char sql[128];
 	char **result;
 	int ret;
 	int cols, row;
-	sqlite_int64 container;
+	sqlite_int64 objectID, parentID;
 
 	if( strstr(class, "imageItem") )
 	{
@@ -144,9 +140,8 @@ insert_containers(const char * name, const char *path, const char * refID, const
 		static struct virtual_item last_camdate;
 		static sqlite_int64 last_all_objectID = 0;
 
-		asprintf(&sql, "SELECT DATE, CREATOR from DETAILS where ID = %lu", detailID);
+		snprintf(sql, sizeof(sql), "SELECT DATE, CREATOR from DETAILS where ID = %lld", detailID);
 		ret = sql_get_table(db, sql, &result, &row, &cols);
-		free(sql);
 		if( ret == SQLITE_OK )
 		{
 			date = result[2];
@@ -169,16 +164,16 @@ insert_containers(const char * name, const char *path, const char * refID, const
 		}
 		else
 		{
-			container = insert_container(date_taken, "3$12", NULL, "album.photoAlbum", NULL, NULL, NULL);
-			sprintf(last_date.parentID, "3$12$%llX", container>>32);
-			last_date.objectID = (int)container;
+			insert_container(date_taken, IMAGE_DATE_ID, NULL, "album.photoAlbum", NULL, NULL, NULL, &objectID, &parentID);
+			sprintf(last_date.parentID, IMAGE_DATE_ID"$%"PRIX64, parentID);
+			last_date.objectID = objectID;
 			strcpy(last_date.name, date_taken);
 			//DEBUG DPRINTF(E_DEBUG, L_SCANNER, "Creating cached date item: %s/%s/%X\n", last_date.name, last_date.parentID, last_date.objectID);
 		}
 		sql_exec(db, "INSERT into OBJECTS"
 		             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 		             "VALUES"
-		             " ('%s$%X', '%s', '%s', '%s', %lu, %Q)",
+		             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
 		             last_date.parentID, last_date.objectID, last_date.parentID, refID, class, detailID, name);
 
 		if( cam )
@@ -192,8 +187,8 @@ insert_containers(const char * name, const char *path, const char * refID, const
 		}
 		if( !valid_cache || strcmp(camera, last_cam.name) != 0 )
 		{
-			container = insert_container(camera, "3$13", NULL, "storageFolder", NULL, NULL, NULL);
-			sprintf(last_cam.parentID, "3$13$%llX", container>>32);
+			insert_container(camera, IMAGE_CAMERA_ID, NULL, "storageFolder", NULL, NULL, NULL, &objectID, &parentID);
+			sprintf(last_cam.parentID, IMAGE_CAMERA_ID"$%"PRIX64, parentID);
 			strncpy(last_cam.name, camera, 255);
 			last_camdate.name[0] = '\0';
 		}
@@ -204,33 +199,32 @@ insert_containers(const char * name, const char *path, const char * refID, const
 		}
 		else
 		{
-			container = insert_container(date_taken, last_cam.parentID, NULL, "album.photoAlbum", NULL, NULL, NULL);
-			sprintf(last_camdate.parentID, "%s$%llX", last_cam.parentID, container>>32);
-			last_camdate.objectID = (int)container;
+			insert_container(date_taken, last_cam.parentID, NULL, "album.photoAlbum", NULL, NULL, NULL, &objectID, &parentID);
+			sprintf(last_camdate.parentID, "%s$%"PRIX64, last_cam.parentID, parentID);
+			last_camdate.objectID = objectID;
 			strcpy(last_camdate.name, date_taken);
 			//DEBUG DPRINTF(E_DEBUG, L_SCANNER, "Creating cached camdate item: %s/%s/%s/%X\n", camera, last_camdate.name, last_camdate.parentID, last_camdate.objectID);
 		}
 		sql_exec(db, "INSERT into OBJECTS"
 		             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 		             "VALUES"
-		             " ('%s$%X', '%s', '%s', '%s', %lu, %Q)",
+		             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
 		             last_camdate.parentID, last_camdate.objectID, last_camdate.parentID, refID, class, detailID, name);
 		/* All Images */
 		if( !last_all_objectID )
 		{
-			last_all_objectID = get_next_available_id("OBJECTS", "3$11");
+			last_all_objectID = get_next_available_id("OBJECTS", IMAGE_ALL_ID);
 		}
 		sql_exec(db, "INSERT into OBJECTS"
 		             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 		             "VALUES"
-		             " ('3$11$%llX', '3$11', '%s', '%s', %lu, %Q)",
+		             " ('"IMAGE_ALL_ID"$%"PRIX64"', '"IMAGE_ALL_ID"', '%s', '%s', %"PRId64", %Q)",
 		             last_all_objectID++, refID, class, detailID, name);
 	}
 	else if( strstr(class, "audioItem") )
 	{
-		asprintf(&sql, "SELECT ALBUM, CREATOR, GENRE, ALBUM_ART from DETAILS where ID = %lu", detailID);
+		snprintf(sql, sizeof(sql), "SELECT ALBUM, ARTIST, GENRE, ALBUM_ART from DETAILS where ID = %lld", detailID);
 		ret = sql_get_table(db, sql, &result, &row, &cols);
-		free(sql);
 		if( ret != SQLITE_OK )
 			return;
 		if( !row )
@@ -259,29 +253,29 @@ insert_containers(const char * name, const char *path, const char * refID, const
 			else
 			{
 				strcpy(last_album.name, album);
-				container = insert_container(album, "1$7", NULL, "album.musicAlbum", artist, genre, album_art);
-				sprintf(last_album.parentID, "1$7$%llX", container>>32);
-				last_album.objectID = (int)container;
+				insert_container(album, MUSIC_ALBUM_ID, NULL, "album.musicAlbum", artist, genre, album_art, &objectID, &parentID);
+				sprintf(last_album.parentID, MUSIC_ALBUM_ID"$%llX", parentID);
+				last_album.objectID = objectID;
 				//DEBUG DPRINTF(E_DEBUG, L_SCANNER, "Creating cached album item: %s/%s/%X\n", last_album.name, last_album.parentID, last_album.objectID);
 			}
 			sql_exec(db, "INSERT into OBJECTS"
 			             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 			             "VALUES"
-			             " ('%s$%X', '%s', '%s', '%s', %lu, %Q)",
+			             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
 			             last_album.parentID, last_album.objectID, last_album.parentID, refID, class, detailID, name);
 		}
 		if( artist )
 		{
 			if( !valid_cache || strcmp(artist, last_artist.name) != 0 )
 			{
-				container = insert_container(artist, "1$6", NULL, "person.musicArtist", NULL, genre, NULL);
-				sprintf(last_artist.parentID, "1$6$%llX", container>>32);
+				insert_container(artist, MUSIC_ARTIST_ID, NULL, "person.musicArtist", NULL, genre, NULL, &objectID, &parentID);
+				sprintf(last_artist.parentID, MUSIC_ARTIST_ID"$%"PRIX64, parentID);
 				strcpy(last_artist.name, artist);
 				last_artistAlbum.name[0] = '\0';
 				/* Add this file to the "- All Albums -" container as well */
-				container = insert_container(_("- All Albums -"), last_artist.parentID, NULL, "storageFolder", artist, genre, NULL);
-				sprintf(last_artistAlbumAll.parentID, "%s$%llX", last_artist.parentID, container>>32);
-				last_artistAlbumAll.objectID = (int)container;
+				insert_container(_("- All Albums -"), last_artist.parentID, NULL, "album", artist, genre, NULL, &objectID, &parentID);
+				sprintf(last_artistAlbumAll.parentID, "%s$%"PRIX64, last_artist.parentID, parentID);
+				last_artistAlbumAll.objectID = objectID;
 			}
 			else
 			{
@@ -294,35 +288,36 @@ insert_containers(const char * name, const char *path, const char * refID, const
 			}
 			else
 			{
-				container = insert_container(album?album:_("Unknown Album"), last_artist.parentID, album?last_album.parentID:NULL, "album.musicAlbum", artist, genre, album_art);
-				sprintf(last_artistAlbum.parentID, "%s$%llX", last_artist.parentID, container>>32);
-				last_artistAlbum.objectID = (int)container;
+				insert_container(album?album:_("Unknown Album"), last_artist.parentID, album?last_album.parentID:NULL,
+				                 "album.musicAlbum", artist, genre, album_art, &objectID, &parentID);
+				sprintf(last_artistAlbum.parentID, "%s$%"PRIX64, last_artist.parentID, parentID);
+				last_artistAlbum.objectID = objectID;
 				strcpy(last_artistAlbum.name, album?album:_("Unknown Album"));
 				//DEBUG DPRINTF(E_DEBUG, L_SCANNER, "Creating cached artist/album item: %s/%s/%X\n", last_artist.name, last_artist.parentID, last_artist.objectID);
 			}
 			sql_exec(db, "INSERT into OBJECTS"
 			             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 			             "VALUES"
-			             " ('%s$%X', '%s', '%s', '%s', %lu, %Q)",
+			             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
 			             last_artistAlbum.parentID, last_artistAlbum.objectID, last_artistAlbum.parentID, refID, class, detailID, name);
 			sql_exec(db, "INSERT into OBJECTS"
 			             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 			             "VALUES"
-			             " ('%s$%X', '%s', '%s', '%s', %lu, %Q)",
+			             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
 			             last_artistAlbumAll.parentID, last_artistAlbumAll.objectID, last_artistAlbumAll.parentID, refID, class, detailID, name);
 		}
 		if( genre )
 		{
 			if( !valid_cache || strcmp(genre, last_genre.name) != 0 )
 			{
-				container = insert_container(genre, "1$5", NULL, "genre.musicGenre", NULL, NULL, NULL);
-				sprintf(last_genre.parentID, "1$5$%llX", container>>32);
+				insert_container(genre, MUSIC_GENRE_ID, NULL, "genre.musicGenre", NULL, NULL, NULL, &objectID, &parentID);
+				sprintf(last_genre.parentID, MUSIC_GENRE_ID"$%"PRIX64, parentID);
 				strcpy(last_genre.name, genre);
 				last_genreArtist.name[0] = '\0';
 				/* Add this file to the "- All Artists -" container as well */
-				container = insert_container(_("- All Artists -"), last_genre.parentID, NULL, "storageFolder", NULL, genre, NULL);
-				sprintf(last_genreArtistAll.parentID, "%s$%llX", last_genre.parentID, container>>32);
-				last_genreArtistAll.objectID = (int)container;
+				insert_container(_("- All Artists -"), last_genre.parentID, NULL, "person", NULL, genre, NULL, &objectID, &parentID);
+				sprintf(last_genreArtistAll.parentID, "%s$%"PRIX64, last_genre.parentID, parentID);
+				last_genreArtistAll.objectID = objectID;
 			}
 			else
 			{
@@ -334,32 +329,33 @@ insert_containers(const char * name, const char *path, const char * refID, const
 			}
 			else
 			{
-				container = insert_container(artist?artist:_("Unknown Artist"), last_genre.parentID, artist?last_artist.parentID:NULL, "person.musicArtist", NULL, genre, NULL);
-				sprintf(last_genreArtist.parentID, "%s$%llX", last_genre.parentID, container>>32);
-				last_genreArtist.objectID = (int)container;
+				insert_container(artist?artist:_("Unknown Artist"), last_genre.parentID, artist?last_artist.parentID:NULL,
+				                 "person.musicArtist", NULL, genre, NULL, &objectID, &parentID);
+				sprintf(last_genreArtist.parentID, "%s$%"PRIX64, last_genre.parentID, parentID);
+				last_genreArtist.objectID = objectID;
 				strcpy(last_genreArtist.name, artist?artist:_("Unknown Artist"));
 				//DEBUG DPRINTF(E_DEBUG, L_SCANNER, "Creating cached genre/artist item: %s/%s/%X\n", last_genreArtist.name, last_genreArtist.parentID, last_genreArtist.objectID);
 			}
 			sql_exec(db, "INSERT into OBJECTS"
 			             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 			             "VALUES"
-			             " ('%s$%X', '%s', '%s', '%s', %lu, %Q)",
+			             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
 			             last_genreArtist.parentID, last_genreArtist.objectID, last_genreArtist.parentID, refID, class, detailID, name);
 			sql_exec(db, "INSERT into OBJECTS"
 			             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 			             "VALUES"
-			             " ('%s$%X', '%s', '%s', '%s', %lu, %Q)",
+			             " ('%s$%"PRIX64"', '%s', '%s', '%s', %"PRId64", %Q)",
 			             last_genreArtistAll.parentID, last_genreArtistAll.objectID, last_genreArtistAll.parentID, refID, class, detailID, name);
 		}
 		/* All Music */
 		if( !last_all_objectID )
 		{
-			last_all_objectID = get_next_available_id("OBJECTS", "1$4");
+			last_all_objectID = get_next_available_id("OBJECTS", MUSIC_ALL_ID);
 		}
 		sql_exec(db, "INSERT into OBJECTS"
 		             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 		             "VALUES"
-		             " ('1$4$%llX', '1$4', '%s', '%s', %lu, %Q)",
+		             " ('"MUSIC_ALL_ID"$%"PRIX64"', '"MUSIC_ALL_ID"', '%s', '%s', %"PRId64", %Q)",
 		             last_all_objectID++, refID, class, detailID, name);
 	}
 	else if( strstr(class, "videoItem") )
@@ -369,12 +365,12 @@ insert_containers(const char * name, const char *path, const char * refID, const
 		/* All Videos */
 		if( !last_all_objectID )
 		{
-			last_all_objectID = get_next_available_id("OBJECTS", "2$8");
+			last_all_objectID = get_next_available_id("OBJECTS", VIDEO_ALL_ID);
 		}
 		sql_exec(db, "INSERT into OBJECTS"
 		             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 		             "VALUES"
-		             " ('2$8$%llX', '2$8', '%s', '%s', %lu, %Q)",
+		             " ('"VIDEO_ALL_ID"$%"PRIX64"', '"VIDEO_ALL_ID"', '%s', '%s', %"PRId64", %Q)",
 		             last_all_objectID++, refID, class, detailID, name);
 		return;
 	}
@@ -389,29 +385,30 @@ insert_containers(const char * name, const char *path, const char * refID, const
 int
 insert_directory(const char * name, const char * path, const char * base, const char * parentID, int objectID)
 {
-	char * sql;
-	int rows, found = 0;
+	int found = 0;
 	sqlite_int64 detailID = 0;
 	char * refID = NULL;
 	char class[] = "container.storageFolder";
-	char * id_buf = NULL;
-	char * parent_buf = NULL;
-	char **result;
-	char *dir = NULL;
+	char *result, *p;
 	static char last_found[256] = "-1";
 
 	if( strcmp(base, BROWSEDIR_ID) != 0 )
-		asprintf(&refID, "%s%s$%X", BROWSEDIR_ID, parentID, objectID);
+	{
+		if( asprintf(&refID, "%s%s$%X", BROWSEDIR_ID, parentID, objectID) == -1 )
+			return 1;
+	}
 
 	if( refID )
 	{
- 		dir = strdup(path);
-		dir = dirname(dir);
-		asprintf(&id_buf, "%s%s$%X", base, parentID, objectID);
-		asprintf(&parent_buf, "%s%s", base, parentID);
+		char id_buf[64], parent_buf[64];
+		char *dir_buf, *dir;
+ 		dir_buf = strdup(path);
+		dir = dirname(dir_buf);
+		snprintf(id_buf, sizeof(id_buf), "%s%s$%X", base, parentID, objectID);
+		snprintf(parent_buf, sizeof(parent_buf), "%s%s", base, parentID);
 		while( !found )
 		{
-			if( strcmp(id_buf, last_found) == 0 )
+			if( valid_cache && strcmp(id_buf, last_found) == 0 )
 				break;
 			if( sql_get_int_field(db, "SELECT count(*) from OBJECTS where OBJECT_ID = '%s'", id_buf) > 0 )
 			{
@@ -419,43 +416,40 @@ insert_directory(const char * name, const char * path, const char * base, const
 				break;
 			}
 			/* Does not exist.  Need to create, and may need to create parents also */
-			sql = sqlite3_mprintf("SELECT DETAIL_ID from OBJECTS where OBJECT_ID = '%s'", refID);
-			if( (sql_get_table(db, sql, &result, &rows, NULL) == SQLITE_OK) && rows )
+			result = sql_get_text_field(db, "SELECT DETAIL_ID from OBJECTS where OBJECT_ID = '%s'", refID);
+			if( result )
 			{
-				detailID = atoi(result[1]);
+				detailID = strtoll(result, NULL, 10);
+				sqlite3_free(result);
 			}
-			sqlite3_free_table(result);
-			sqlite3_free(sql);
 			sql_exec(db, "INSERT into OBJECTS"
 			             " (OBJECT_ID, PARENT_ID, REF_ID, DETAIL_ID, CLASS, NAME) "
 			             "VALUES"
-			             " ('%s', '%s', %Q, '%lld', '%s', '%q')",
-			             id_buf, parent_buf, refID, detailID, class, rindex(dir, '/')+1);
-			if( rindex(id_buf, '$') )
-				*rindex(id_buf, '$') = '\0';
-			if( rindex(parent_buf, '$') )
-				*rindex(parent_buf, '$') = '\0';
-			if( rindex(refID, '$') )
-				*rindex(refID, '$') = '\0';
+			             " ('%s', '%s', %Q, %"PRId64", '%s', '%q')",
+			             id_buf, parent_buf, refID, detailID, class, strrchr(dir, '/')+1);
+			if( (p = strrchr(id_buf, '$')) )
+				*p = '\0';
+			if( (p = strrchr(parent_buf, '$')) )
+				*p = '\0';
+			if( (p = strrchr(refID, '$')) )
+				*p = '\0';
 			dir = dirname(dir);
 		}
 		free(refID);
-		free(parent_buf);
-		free(id_buf);
-		free(dir);
-		return 1;
+		free(dir_buf);
+		return 0;
 	}
 
-	detailID = GetFolderMetadata(name, path, NULL, NULL, NULL);
+	detailID = GetFolderMetadata(name, path, NULL, NULL, find_album_art(path, NULL, 0));
 	sql_exec(db, "INSERT into OBJECTS"
 	             " (OBJECT_ID, PARENT_ID, REF_ID, DETAIL_ID, CLASS, NAME) "
 	             "VALUES"
-	             " ('%s%s$%X', '%s%s', %Q, '%lld', '%s', '%q')",
+	             " ('%s%s$%X', '%s%s', %Q, %"PRId64", '%s', '%q')",
 	             base, parentID, objectID, base, parentID, refID, detailID, class, name);
 	if( refID )
 		free(refID);
 
-	return -1;
+	return 0;
 }
 
 int
@@ -463,7 +457,7 @@ insert_file(char * name, const char * path, const char * parentID, int object)
 {
 	char class[32];
 	char objectID[64];
-	unsigned long int detailID = 0;
+	sqlite3_int64 detailID = 0;
 	char base[8];
 	char * typedir_parentID;
 	int typedir_objectID;
@@ -472,6 +466,8 @@ insert_file(char * name, const char * path, const char * parentID, int object)
 
 	if( is_image(name) )
 	{
+		if( is_album_art(name) )
+			return -1;
 		strcpy(base, IMAGE_DIR_ID);
 		strcpy(class, "item.imageItem.photo");
 		detailID = GetImageMetadata(path, name);
@@ -509,17 +505,17 @@ insert_file(char * name, const char * path, const char * parentID, int object)
 	sql_exec(db, "INSERT into OBJECTS"
 	             " (OBJECT_ID, PARENT_ID, CLASS, DETAIL_ID, NAME) "
 	             "VALUES"
-	             " ('%s', '%s%s', '%s', %lu, '%q')",
+	             " ('%s', '%s%s', '%s', %"PRId64", '%q')",
 	             objectID, BROWSEDIR_ID, parentID, class, detailID, name);
 
 	if( *parentID )
 	{
 		typedir_objectID = 0;
 		typedir_parentID = strdup(parentID);
-		baseid = rindex(typedir_parentID, '$');
+		baseid = strrchr(typedir_parentID, '$');
 		if( baseid )
 		{
-			sscanf(baseid+1, "%X", &typedir_objectID);
+			typedir_objectID = strtol(baseid+1, NULL, 16);
 			*baseid = '\0';
 		}
 		insert_directory(name, path, base, typedir_parentID, typedir_objectID);
@@ -528,7 +524,7 @@ insert_file(char * name, const char * path, const char * parentID, int object)
 	sql_exec(db, "INSERT into OBJECTS"
 	             " (OBJECT_ID, PARENT_ID, REF_ID, CLASS, DETAIL_ID, NAME) "
 	             "VALUES"
-	             " ('%s%s$%X', '%s%s', '%s', '%s', %lu, '%q')",
+	             " ('%s%s$%X', '%s%s', '%s', '%s', %"PRId64", '%q')",
 	             base, parentID, object, base, parentID, objectID, class, detailID, name);
 
 	insert_containers(name, path, objectID, class, detailID);
@@ -539,24 +535,27 @@ int
 CreateDatabase(void)
 {
 	int ret, i;
-	const char * containers[] = {      "0","-1", "root",
-					   "1", "0", _("Music"),
-					 "1$4", "1", _("All Music"),
-					 "1$5", "1", _("Genre"),
-					 "1$6", "1", _("Artist"),
-					 "1$7", "1", _("Album"),
-				  MUSIC_DIR_ID, "1", _("Folders"),
-				MUSIC_PLIST_ID, "1", _("Playlists"),
-					   "2", "0", _("Video"),
-					 "2$8", "2", _("All Video"),
-				  VIDEO_DIR_ID, "2", _("Folders"),
-					   "3", "0", _("Pictures"),
-					"3$11", "3", _("All Pictures"),
-					"3$12", "3", _("Date Taken"),
-					"3$13", "3", _("Camera"),
-				  IMAGE_DIR_ID, "3", _("Folders"),
-					  "64", "0", _("Browse Folders"),
-					0 };
+	const char * containers[] = { "0","-1",   "root",
+	                         MUSIC_ID, "0", _("Music"),
+	                     MUSIC_ALL_ID, MUSIC_ID, _("All Music"),
+	                   MUSIC_GENRE_ID, MUSIC_ID, _("Genre"),
+	                  MUSIC_ARTIST_ID, MUSIC_ID, _("Artist"),
+	                   MUSIC_ALBUM_ID, MUSIC_ID, _("Album"),
+	                     MUSIC_DIR_ID, MUSIC_ID, _("Folders"),
+	                   MUSIC_PLIST_ID, MUSIC_ID, _("Playlists"),
+
+	                         VIDEO_ID, "0", _("Video"),
+	                     VIDEO_ALL_ID, VIDEO_ID, _("All Video"),
+	                     VIDEO_DIR_ID, VIDEO_ID, _("Folders"),
+
+	                         IMAGE_ID, "0", _("Pictures"),
+	                     IMAGE_ALL_ID, IMAGE_ID, _("All Pictures"),
+	                    IMAGE_DATE_ID, IMAGE_ID, _("Date Taken"),
+	                  IMAGE_CAMERA_ID, IMAGE_ID, _("Camera"),
+	                     IMAGE_DIR_ID, IMAGE_ID, _("Folders"),
+
+	                     BROWSEDIR_ID, "0", _("Browse Folders"),
+			0 };
 
 	ret = sql_exec(db, "CREATE TABLE OBJECTS ( "
 					"ID INTEGER PRIMARY KEY AUTOINCREMENT, "
@@ -573,25 +572,26 @@ CreateDatabase(void)
 					"ID INTEGER PRIMARY KEY AUTOINCREMENT, "
 					"PATH TEXT DEFAULT NULL, "
 					"SIZE INTEGER, "
+					"TIMESTAMP INTEGER, "
 					"TITLE TEXT COLLATE NOCASE, "
 					"DURATION TEXT, "
 					"BITRATE INTEGER, "
 					"SAMPLERATE INTEGER, "
+					"CREATOR TEXT COLLATE NOCASE, "
 					"ARTIST TEXT COLLATE NOCASE, "
 					"ALBUM TEXT COLLATE NOCASE, "
 					"GENRE TEXT COLLATE NOCASE, "
 					"COMMENT TEXT, "
 					"CHANNELS INTEGER, "
+					"DISC INTEGER, "
 					"TRACK INTEGER, "
 					"DATE DATE, "
 					"RESOLUTION TEXT, "
 					"THUMBNAIL BOOL DEFAULT 0, "
-					"CREATOR TEXT COLLATE NOCASE, "
-					"DLNA_PN TEXT, "
-					"MIME TEXT, "
 					"ALBUM_ART INTEGER DEFAULT 0, "
-					"DISC INTEGER, "
-					"TIMESTAMP INTEGER"
+					"ROTATION INTEGER, "
+					"DLNA_PN TEXT, "
+					"MIME TEXT"
 					")");
 	if( ret != SQLITE_OK )
 		goto sql_failed;
@@ -607,6 +607,12 @@ CreateDatabase(void)
 					")");
 	if( ret != SQLITE_OK )
 		goto sql_failed;
+	ret = sql_exec(db, "CREATE TABLE BOOKMARKS ("
+					"ID INTEGER PRIMARY KEY, "
+					"SEC INTEGER"
+					")");
+	if( ret != SQLITE_OK )
+		goto sql_failed;
 	ret = sql_exec(db, "CREATE TABLE PLAYLISTS ("
 					"ID INTEGER PRIMARY KEY AUTOINCREMENT, "
 					"NAME TEXT NOT NULL, "
@@ -629,8 +635,8 @@ CreateDatabase(void)
 	{
 		ret = sql_exec(db, "INSERT into OBJECTS (OBJECT_ID, PARENT_ID, DETAIL_ID, CLASS, NAME)"
 		                   " values "
-		                   "('%s', '%s', %lld, 'container.storageFolder', '%s')",
-		                   containers[i], containers[i+1], GetFolderMetadata(containers[i+2], NULL, NULL, NULL, NULL), containers[i+2]);
+		                   "('%s', '%s', %lld, 'container.storageFolder', '%q')",
+		                   containers[i], containers[i+1], GetFolderMetadata(containers[i+2], NULL, NULL, NULL, 0), containers[i+2]);
 		if( ret != SQLITE_OK )
 			goto sql_failed;
 	}
@@ -749,9 +755,13 @@ ScanDirectory(const char * dir, const char * parent, enum media_types dir_type)
 
 	for (i=0; i < n; i++)
 	{
+#if !USE_FORK
+		if( quitting )
+			break;
+#endif
 		type = TYPE_UNKNOWN;
 		sprintf(full_path, "%s/%s", dir, namelist[i]->d_name);
-		name = escape_tag(namelist[i]->d_name);
+		name = escape_tag(namelist[i]->d_name, 1);
 		if( namelist[i]->d_type == DT_DIR )
 		{
 			type = TYPE_DIR;
@@ -764,19 +774,18 @@ ScanDirectory(const char * dir, const char * parent, enum media_types dir_type)
 		{
 			type = resolve_unknown_type(full_path, dir_type);
 		}
-		if( type == TYPE_DIR )
+		if( (type == TYPE_DIR) && (access(full_path, R_OK|X_OK) == 0) )
 		{
-			insert_directory(name?name:namelist[i]->d_name, full_path, BROWSEDIR_ID, (parent ? parent:""), i+startID);
+			insert_directory(name, full_path, BROWSEDIR_ID, (parent ? parent:""), i+startID);
 			sprintf(parent_id, "%s$%X", (parent ? parent:""), i+startID);
 			ScanDirectory(full_path, parent_id, dir_type);
 		}
-		else if( type == TYPE_FILE )
+		else if( type == TYPE_FILE && (access(full_path, R_OK) == 0) )
 		{
-			if( insert_file(name?name:namelist[i]->d_name, full_path, (parent ? parent:""), i+startID) == 0 )
+			if( insert_file(name, full_path, (parent ? parent:""), i+startID) == 0 )
 				fileno++;
 		}
-		if( name )
-			free(name);
+		free(name);
 		free(namelist[i]);
 	}
 	free(namelist);
@@ -794,6 +803,7 @@ void
 start_scanner()
 {
 	struct media_dir_s * media_path = media_dirs;
+	char name[PATH_MAX];
 
 	if (setpriority(PRIO_PROCESS, 0, 15) == -1)
 		DPRINTF(E_WARN, L_INOTIFY,  "Failed to reduce scanner thread priority\n");
@@ -806,6 +816,8 @@ start_scanner()
 	freopen("/dev/null", "a", stderr);
 	while( media_path )
 	{
+		strncpy(name, media_path->path, sizeof(name));
+		GetFolderMetadata(basename(name), media_path->path, NULL, NULL, 0);
 		ScanDirectory(media_path->path, NULL, media_path->type);
 		media_path = media_path->next;
 	}
@@ -820,8 +832,16 @@ start_scanner()
 	 * client that uses UPnPSearch on large containers). */
 	sql_exec(db, "create INDEX IDX_SEARCH_OPT ON OBJECTS(OBJECT_ID, CLASS, DETAIL_ID);");
 
-	fill_playlists();
+	if( GETFLAG(NO_PLAYLIST_MASK) )
+	{
+		DPRINTF(E_WARN, L_SCANNER, "Playlist creation disabled\n");	  
+	}
+	else
+	{
+		fill_playlists();
+	}
 
+	DPRINTF(E_DEBUG, L_SCANNER, "Initial file scan completed\n", DB_VERSION);
 	//JM: Set up a db version number, so we know if we need to rebuild due to a new structure.
 	sql_exec(db, "pragma user_version = %d;", DB_VERSION);
 }
