@@ -1,10 +1,31 @@
-/* $Id: upnpdescgen.c,v 1.15 2009/07/17 22:54:31 jmaggard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2008 Thomas Bernard 
- * This software is subject to the conditions detailed
- * in the LICENCE file provided within the distribution */
-
+ *
+ * Copyright (c) 2006-2008, Thomas Bernard
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *     * Redistributions of source code must retain the above copyright
+ *       notice, this list of conditions and the following disclaimer.
+ *     * Redistributions in binary form must reproduce the above copyright
+ *       notice, this list of conditions and the following disclaimer in the
+ *       documentation and/or other materials provided with the distribution.
+ *     * The name of the author may not be used to endorse or promote products
+ *       derived from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -95,37 +116,46 @@ static const char xmlver[] =
 static const char root_service[] =
 	"scpd xmlns=\"urn:schemas-upnp-org:service-1-0\"";
 static const char root_device[] = 
-	"root xmlns=\"urn:schemas-upnp-org:device-1-0\"";
+	"root xmlns=\"urn:schemas-upnp-org:device-1-0\""
+#if PNPX
+	" xmlns:pnpx=\"http://schemas.microsoft.com/windows/pnpx/2005/11\""
+	" xmlns:df=\"http://schemas.microsoft.com/windows/2008/09/devicefoundation\""
+#endif
+	;
 
-/* root Description of the UPnP Device 
- * fixed to match UPnP_IGD_InternetGatewayDevice 1.0.pdf 
- * presentationURL is only "recommended" but the router doesn't appears
- * in "Network connections" in Windows XP if it is not present. */
+/* root Description of the UPnP Device */
 static const struct XMLElt rootDesc[] =
 {
 	{root_device, INITHELPER(1,2)},
 	{"specVersion", INITHELPER(3,2)},
-	{"device", INITHELPER(5,14)},
+	{"device", INITHELPER(5,(14+PNPX))},
 	{"/major", "1"},
 	{"/minor", "0"},
 	{"/deviceType", "urn:schemas-upnp-org:device:MediaServer:1"},
+#if PNPX == 5
+	{"/pnpx:X_hardwareId", pnpx_hwid},
+	{"/pnpx:X_compatibleId", "MS_DigitalMediaDeviceClass_DMS_V001"},
+	{"/pnpx:X_deviceCategory", "MediaDevices"},
+	{"/df:X_deviceCategory", "Multimedia.DMS"},
+	{"/microsoft:magicPacketWakeSupported xmlns:microsoft=\"urn:schemas-microsoft-com:WMPNSS-1-0\"", "0"},
+#endif
 	{"/friendlyName", friendly_name},	/* required */
 	{"/manufacturer", ROOTDEV_MANUFACTURER},		/* required */
 	{"/manufacturerURL", ROOTDEV_MANUFACTURERURL},	/* optional */
 	{"/modelDescription", ROOTDEV_MODELDESCRIPTION}, /* recommended */
-	{"/modelName", ROOTDEV_MODELNAME},	/* required */
+	{"/modelName", modelname},	/* required */
 	{"/modelNumber", modelnumber},
 	{"/modelURL", ROOTDEV_MODELURL},
 	{"/serialNumber", serialnumber},
 	{"/UDN", uuidvalue},	/* required */
 	{"/dlna:X_DLNADOC xmlns:dlna=\"urn:schemas-dlna-org:device-1-0\"", "DMS-1.50"},
 	{"/presentationURL", presentationurl},	/* recommended */
-	{"iconList", INITHELPER(19,4)},
-	{"serviceList", INITHELPER(43,3)},
-	{"icon", INITHELPER(23,5)},
-	{"icon", INITHELPER(28,5)},
-	{"icon", INITHELPER(33,5)},
-	{"icon", INITHELPER(38,5)},
+	{"iconList", INITHELPER((19+PNPX),4)},
+	{"serviceList", INITHELPER((43+PNPX),3)},
+	{"icon", INITHELPER((23+PNPX),5)},
+	{"icon", INITHELPER((28+PNPX),5)},
+	{"icon", INITHELPER((33+PNPX),5)},
+	{"icon", INITHELPER((38+PNPX),5)},
 	{"/mimetype", "image/png"},
 	{"/width", "48"},
 	{"/height", "48"},
@@ -146,9 +176,9 @@ static const struct XMLElt rootDesc[] =
 	{"/height", "120"},
 	{"/depth", "24"},
 	{"/url", "/icons/lrg.jpg"},
-	{"service", INITHELPER(46,5)},
-	{"service", INITHELPER(51,5)},
-	{"service", INITHELPER(56,5)},
+	{"service", INITHELPER((46+PNPX),5)},
+	{"service", INITHELPER((51+PNPX),5)},
+	{"service", INITHELPER((56+PNPX),5)},
 	{"/serviceType", "urn:schemas-upnp-org:service:ContentDirectory:1"},
 	{"/serviceId", "urn:upnp-org:serviceId:ContentDirectory"},
 	{"/controlURL", CONTENTDIRECTORY_CONTROLURL},
@@ -496,6 +526,7 @@ static const struct serviceDesc scpdX_MS_MediaReceiverRegistrar =
 static char *
 strcat_str(char * str, int * len, int * tmplen, const char * s2)
 {
+	char *p;
 	int s2len;
 	s2len = (int)strlen(s2);
 	if(*tmplen <= (*len + s2len))
@@ -504,7 +535,17 @@ strcat_str(char * str, int * len, int * tmplen, const char * s2)
 			*tmplen += 256;
 		else
 			*tmplen += s2len + 1;
-		str = (char *)realloc(str, *tmplen);
+		p = realloc(str, *tmplen);
+		if (!p)
+		{
+			if(s2len < 256)
+				*tmplen -= 256;
+			else
+				*tmplen -= s2len + 1;
+			return str;
+		}
+		else
+			str = p;
 	}
 	/*strcpy(str + *len, s2); */
 	memcpy(str + *len, s2, s2len + 1);
@@ -518,10 +559,18 @@ strcat_str(char * str, int * len, int * tmplen, const char * s2)
 static char *
 strcat_char(char * str, int * len, int * tmplen, char c)
 {
+	char *p;
 	if(*tmplen <= (*len + 1))
 	{
 		*tmplen += 256;
-		str = (char *)realloc(str, *tmplen);
+		p = (char *)realloc(str, *tmplen);
+		if (!p)
+		{
+			*tmplen -= 256;
+			return str;
+		}
+		else
+			str = p;
 	}
 	str[*len] = c;
 	(*len)++;
@@ -625,18 +674,40 @@ genRootDesc(int * len)
 {
 	char * str;
 	int tmplen;
-	tmplen = 2048;
+	tmplen = 2560;
 	str = (char *)malloc(tmplen);
 	if(str == NULL)
 		return NULL;
 	* len = strlen(xmlver);
-	/*strcpy(str, xmlver); */
 	memcpy(str, xmlver, *len + 1);
 	str = genXML(str, len, &tmplen, rootDesc);
 	str[*len] = '\0';
 	return str;
 }
 
+char *
+genRootDescSamsung(int * len)
+{
+	char * str;
+	int tmplen;
+	struct XMLElt samsungRootDesc[sizeof(rootDesc)/sizeof(struct XMLElt)];
+	tmplen = 2560;
+	str = (char *)malloc(tmplen);
+	if(str == NULL)
+		return NULL;
+	* len = strlen(xmlver);
+	memcpy(str, xmlver, *len + 1);
+	/* Replace the optional modelURL and manufacturerURL fields with Samsung foo */
+	memcpy(&samsungRootDesc, &rootDesc, sizeof(rootDesc));
+	samsungRootDesc[8+PNPX].eltname = "/sec:ProductCap";
+	samsungRootDesc[8+PNPX].data = "smi,DCM10,getMediaInfo.sec,getCaptionInfo.sec";
+	samsungRootDesc[12+PNPX].eltname = "/sec:X_ProductCap";
+	samsungRootDesc[12+PNPX].data = "smi,DCM10,getMediaInfo.sec,getCaptionInfo.sec";
+	str = genXML(str, len, &tmplen, samsungRootDesc);
+	str[*len] = '\0';
+	return str;
+}
+
 /* genServiceDesc() :
  * Generate service description with allowed methods and 
  * related variables. */
