@@ -61,7 +61,7 @@
 time_t startup_time = 0;
 
 struct runtime_vars_s runtime_vars;
-int runtime_flags = INOTIFY_MASK;
+uint32_t runtime_flags = INOTIFY_MASK;
 
 const char * pidfilename = "/var/run/minidlna.pid";
 
@@ -80,9 +80,11 @@ char presentationurl[PRESENTATIONURL_MAX_LEN];
 int n_lan_addr = 0;
 struct lan_addr_s lan_addr[MAX_LAN_ADDR];
 
+/* Path of the Unix socket used to communicate with MiniSSDPd */
+const char * minissdpdsocketpath = "/var/run/minissdpd.sock";
+
 /* UPnP-A/V [DLNA] */
 sqlite3 * db;
-char dlna_no_conv[] = "DLNA.ORG_OP=01;DLNA.ORG_CI=0";
 char friendly_name[FRIENDLYNAME_MAX_LEN];
 char db_path[PATH_MAX] = {'\0'};
 char log_path[PATH_MAX] = {'\0'};
