@@ -45,6 +45,13 @@ strcatf(struct string_s *str, const char *fmt, ...)
 	return ret;
 }
 
+inline void
+strncpyt(char *dst, const char *src, size_t len)
+{
+	strncpy(dst, src, len);
+	dst[len-1] = '\0';
+}
+
 int
 ends_with(const char * haystack, const char * needle)
 {
@@ -62,16 +69,32 @@ ends_with(const char * haystack, const char * needle)
 char *
 trim(char *str)
 {
-        if (!str)
-                return(NULL);
-        int i;
-        for (i=0; i <= strlen(str) && (isspace(str[i]) || str[i] == '"'); i++) {
+	int i;
+	int len;
+
+	if (!str)
+		return(NULL);
+
+	len = strlen(str);
+	for (i=len-1; i >= 0 && isspace(str[i]); i--)
+	{
+		str[i] = '\0';
+		len--;
+	}
+	while (isspace(*str))
+	{
+		str++;
+		len--;
+	}
+
+	if (str[0] == '"' && str[len-1] == '"')
+	{
+		str[0] = '\0';
+		str[len-1] = '\0';
 		str++;
 	}
-        for (i=(strlen(str)-1); i >= 0 && (isspace(str[i]) || str[i] == '"'); i--) {
-                str[i] = '\0';
-        }
-        return str;
+
+	return str;
 }
 
 /* Find the first occurrence of p in s, where s is terminated by t */
@@ -83,7 +106,7 @@ strstrc(const char *s, const char *p, const char t)
 
 	endptr = strchr(s, t);
 	if (!endptr)
-		return NULL;
+		return strstr(s, p);
 
 	plen = strlen(p);
 	slen = endptr - s;
@@ -98,6 +121,29 @@ strstrc(const char *s, const char *p, const char t)
 	return NULL;
 } 
 
+char *
+strcasestrc(const char *s, const char *p, const char t)
+{
+	char *endptr;
+	size_t slen, plen;
+
+	endptr = strchr(s, t);
+	if (!endptr)
+		return strcasestr(s, p);
+
+	plen = strlen(p);
+	slen = endptr - s;
+	while (slen >= plen)
+	{
+		if (*s == *p && strncasecmp(s+1, p+1, plen-1) == 0)
+			return (char*)s;
+		s++;
+		slen--;
+	}
+
+	return NULL;
+} 
+
 char *
 modifyString(char * string, const char * before, const char * after, short like)
 {
@@ -159,12 +205,13 @@ escape_tag(const char *tag, int force_alloc)
 {
 	char *esc_tag = NULL;
 
-	if( strchr(tag, '&') || strchr(tag, '<') || strchr(tag, '>') )
+	if( strchr(tag, '&') || strchr(tag, '<') || strchr(tag, '>') || strchr(tag, '"') )
 	{
 		esc_tag = strdup(tag);
 		esc_tag = modifyString(esc_tag, "&", "&amp;amp;", 0);
 		esc_tag = modifyString(esc_tag, "<", "&amp;lt;", 0);
 		esc_tag = modifyString(esc_tag, ">", "&amp;gt;", 0);
+		esc_tag = modifyString(esc_tag, "\"", "&amp;quot;", 0);
 	}
 	else if( force_alloc )
 		esc_tag = strdup(tag);
@@ -191,7 +238,7 @@ make_dir(char * path, mode_t mode)
 	struct stat st;
 
 	do {
-		c = 0;
+		c = '\0';
 
 		/* Bypass leading non-'/'s and then subsequent '/'s. */
 		while (*s) {
@@ -200,7 +247,7 @@ make_dir(char * path, mode_t mode)
 					++s;
 				} while (*s == '/');
 				c = *s;     /* Save the current char */
-				*s = 0;     /* and replace it with nul. */
+				*s = '\0';     /* and replace it with nul. */
 				break;
 			}
 			++s;
@@ -209,9 +256,11 @@ make_dir(char * path, mode_t mode)
 		if (mkdir(path, mode) < 0) {
 			/* If we failed for any other reason than the directory
 			 * already exists, output a diagnostic and return -1.*/
-			if (errno != EEXIST
-			    || (stat(path, &st) < 0 || !S_ISDIR(st.st_mode))) {
-				break;
+			if (errno != EEXIST || (stat(path, &st) < 0 || !S_ISDIR(st.st_mode))) {
+				DPRINTF(E_WARN, L_GENERAL, "make_dir: cannot create directory '%s'\n", path);
+				if (c)
+					*s = c;
+				return -1;
 			}
 		}
 	        if (!c)
@@ -221,9 +270,21 @@ make_dir(char * path, mode_t mode)
 		*s = c;
 
 	} while (1);
+}
 
-	DPRINTF(E_WARN, L_GENERAL, "make_dir: cannot create directory '%s'\n", path);
-	return -1;
+/* Simple, efficient hash function from Daniel J. Bernstein */
+unsigned int
+DJBHash(const char *str, int len)
+{
+	unsigned int hash = 5381;
+	unsigned int i = 0;
+
+	for(i = 0; i < len; str++, i++)
+	{
+		hash = ((hash << 5) + hash) + (*str);
+	}
+
+	return hash;
 }
 
 int
